#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2016, 2018, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use FindBin;
use List::Util 'max','min';
use Math::Libm 'hypot';
use Math::Matrix;
use Math::PlanePath::FractionsTree;
use Math::PlanePath::RationalsTree;
use Math::Geometry::Planar;
use Memoize 'memoize';
use Math::PlanePath::Base::Digits 'bit_split_lowtohigh', 'digit_join_lowtohigh';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use lib::abs "$FindBin::Bin/../../dragon/tools";
use MyRationals ':all';
$|=1;

# uncomment this to run the ### lines
use Smart::Comments;

# p' = a.-b+b.1    p=abs(b) q=1
# q' = c.-b


{
  # Stern diatomic mod 3 = A071911
  # Math:NumSeq::Stern Diatomic
  # bits of n high to low
  #   p,q ->  p+=q   if 1
  #           q+=p   if 0

  require FLAT;
  my $f = FLAT::DFA->new;
  $f->add_states(9);
  my $want = 0;
  my @pq_to_state_table = (0, 1,2, 3,4,5, 6,7,8);
  my $pq_to_state = sub {
    my ($p,$q) = @_;
    return $pq_to_state_table[3*$p+$q];
  };
  foreach my $p (0..2) {
    foreach my $q (0..2) {
      my $from = $pq_to_state->($p,$q);
      if ($p==$want) { $f->set_accepting($from); }
      my $s = ($p+$q) % 3;
      $f->add_transition($from, $pq_to_state->($s,$q), 1);
      $f->add_transition($from, $pq_to_state->($p,$s), 0);
    }
  }
  $f->set_starting($pq_to_state->(0,1));
  require MyFLAT;
  # $f = $f->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize;
  $f->MyFLAT::view;
  exit 0;
}
{
  # Stern diatomic mod 3 = A071911
  # Math:NumSeq::Stern Diatomic
  # bits of n high to low
  #   p,q ->  p+=q   if 0
  #           q+=p   if 1

  my %table;
  my %accepting;
  foreach my $p (0..2) {
    foreach my $q (0..2) {
      my $from = "$p$q";
      my $s = ($p+$q) % 3;
      $table{$from}->{1} = "$s$q";
      $table{$from}->{0} = "$p$s";
      $accepting{$from} = "m$p";
    }
  }
  ### %accepting
  require MyFSM;
  my $fsm = MyFSM->new(table => \%table,
                       initial => '01',
                       accepting => \%accepting,
                       title => 'Stern Diatomic Mod 3',
                       flow => 'east',
                      );
  $fsm->delete_unreached_states;
  $fsm = $fsm->reverse (verbose => 1);
  $fsm->show;
  ### $fsm
  # $fsm->simplify (verbose => 1);
  # $fsm->renumber_states;
  $fsm->view;
  exit 0;
}
{
  # Stern diatomic mod 3 = A071911

  # count matched of n bits
  # not in OEIS: 1,1,3,5,7,17,35,61,127,265,507,1013,2071,4097,8147,16429

  # x - 2        roots 2.00000
  # x - 1        roots 1.00000
  # x^2 + x + 2  roots  -0.500000+I*1.32288,  -0.500000+I*-1.32288
  # sqrt(7)/2
  # -1/2/(1 - x)
  # + 1/2/(1 - 2*x)
  # - 1/2*x/(1 + x + 2*x^2)

  # Dijkstra regex  0* 1 (.01*0 | .10*1)* . 1 0*
  # doesn't include 0
  require FLAT;
  require MyFLAT;
  my $f = FLAT::Regex->new(' 0* 1 ((0|1) 01*0 | (0|1) 10*1)* (0|1) 1 0* |0* ')
    ->MyFLAT::minimize;
   $f->MyFLAT::view;

  foreach my $len (1 .. 16) {
    my $count = 0;
    foreach my $n (0 .. 2**$len-1) {
      my $str = sprintf '%b', $n;
      if ($f->contains($str)) {
        $count++;
      }
    }
    print $count,",";
  }
  exit 0;
}
{
  # XY convex hull print

  require MyXYConvexHull;
  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
  foreach my $depth (0 .. 10) {
    print "depth=$depth\n";
    my ($n_lo,$n_hi) = $path->tree_depth_to_n_range($depth);
    my $n1 = $path->n_start;
    my $n2 = $n_hi;
    $n1 = $n_lo;
    my @points = map {[$path->n_to_xy($_)]} $n1 .. $n2;
    my $h = MyXYConvexHull->new_from_points (@points);
    $h->print;
  }
  exit 0;
}
{
  # XY convex hull

  #    *--*
  #    |
  # *--.
  #    |
  #    *
  # total - area = 2*F(d) - 1

  # depth=7 1..255  t=725 a=658
  # depth=8 1..511  t=1854 a=1745
  # depth=9 1..1023  t=4779 a=4602
  # depth=10 1..2047  t=12379 a=12092
  # depth=11 1..4095  t=32208 a=31743
  # depth=12 1..8191  t=83975 a=83222
  # depth=13 1..16383  t=219281 a=218062
  # depth=14 1..32767  t=573102 a=571129
  # depth=15 1..65535  t=1498779 a=1495586
  # depth=16 1..131071  t=3921057 a=3915890

  # 1745 4602 12092 31743 83222 218062 571129 1495586 3915890

  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
  # require Math::PlanePath::DragonCurve;
  # $path = Math::PlanePath::DragonCurve->new;
  # require Math::PlanePath::R5DragonCurve;
  # $path = Math::PlanePath::R5DragonCurve->new;
  my $type;
  $type = 'row';
  $type = 'whole';
  my $n_start = $path->n_start;
  my $prev_log = 0;
  print "$type\n";
  my @values_array;
  foreach my $depth (0 .. 24) {
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
    if (! $path->is_tree) {
      ($n_row, $n_end) = $path->level_to_n_range($depth);
      # $n_end -= 1;
    }
    my @xy;
    my $x_max = 1;
    my $y_max = 1;
    my $n_from = ($type eq 'row' ? $n_row : $n_start);
    my $x_offset = 64+240;
    my $y_offset = 64+240;
    # $x_offset = 16;
    # $y_offset = 16;
    foreach my $n ($n_from .. $n_end) {
      my ($x,$y) = $path->n_to_xy($n);
      # print "$x,$y\n";
      $x += $x_offset;
      $y += $y_offset;
      $xy[$x][$y] = '@';
      $x_max = max($x_max, $x);
      $y_max = max($y_max, $y);
    }
    for (;;) {
      my $more = 0;

      foreach my $y (1 .. $y_max) {
        my ($min, $max);
        foreach my $x (1 .. $x_max) {
          if ($xy[$x][$y]) {
            $max = $x;
            if (! $min) { $min = $x; }
          }
        }
        if (defined $min) {
          foreach my $x ($min .. $max) {
            if (! $xy[$x][$y]) {
              $xy[$x][$y] = '+';
              $more = 1;
            }
          }
        }
      }

      foreach my $x (1 .. $x_max) {
        my ($min, $max);
        foreach my $y (1 .. $y_max) {
          if ($xy[$x][$y]) {
            $max = $y;
            if (! $min) { $min = $y; }
          }
        }
        if (defined $min) {
          foreach my $y ($min .. $max) {
            if (! $xy[$x][$y]) {
              $xy[$x][$y] = '+';
              $more = 1;
            }
          }
        }
      }

      last unless $more;
    }

    my $total = 0;
    foreach my $y (1 .. $y_max) {
      foreach my $x (1 .. $x_max) {
        if ($xy[$x][$y]) {
          $total++;
        }
      }
    }

    my $area = 0;
    foreach my $y (1 .. $y_max) {
      foreach my $x (1 .. $x_max) {
        if ($xy[$x][$y] && $xy[$x][$y+1]
            && $xy[$x+1][$y] && $xy[$x+1][$y+1]) {
          $area++;
        }
      }
    }

    my $num_left_sides = 0;
    {
      my $prev_x = undef;
      foreach my $y (0 .. $y_max) {
        foreach my $x (1 .. $x_max+2) {
          if (! $xy[$x-1][$y] && $xy[$x][$y]) {
            if (! defined $prev_x || $x != $prev_x) {
              $num_left_sides++;
              # print "left side at y=$y  x=$x prev_x=",$prev_x//'undef',"\n";
            }
            $prev_x = $x;
            last;
          }
        }
      }
    }

    my $num_right_sides = 0;
    {
      my $prev_x = undef;
      foreach my $y (0 .. $y_max) {
        foreach my $x (reverse 1 .. $x_max+2) {
          if ($xy[$x-1][$y] && ! $xy[$x][$y]) {
            if (! defined $prev_x || $x != $prev_x) {
              $num_right_sides++;
              # print "right side at y=$y  x=$x prev_x=",$prev_x//'undef',"\n";
            }
            $prev_x = $x;
            last;
          }
        }
      }
    }

    my $num_top_sides = 0;
    {
      my $prev_y = undef;
      foreach my $x (0 .. $x_max) {
        foreach my $y (1 .. $y_max+2) {
          if (! $xy[$x][$y-1] && $xy[$x][$y]) {
            if (! defined $prev_y || $y != $prev_y) {
              $num_top_sides++;
            }
            $prev_y = $y;
            last;
          }
        }
      }
    }

    my $num_bottom_sides = 0;
    {
      my $prev_y = undef;
      foreach my $x (0 .. $x_max) {
        foreach my $y (reverse 1 .. $y_max+2) {
          if (! $xy[$x][$y-1] && $xy[$x][$y]) {
            if (! defined $prev_y || $y != $prev_y) {
              $num_bottom_sides++;
            }
            $prev_y = $y;
            last;
          }
        }
      }
    }

    my $ta = $total - $area;
    print "depth=$depth $n_from..$n_end  t=$total a=$area s=$num_left_sides,$num_right_sides $num_top_sides,$num_bottom_sides\n";

    push @{$values_array[0]}, $total;
    push @{$values_array[1]}, $area;
    push @{$values_array[2]}, $ta;
    push @{$values_array[3]}, $num_left_sides;
    push @{$values_array[4]}, $num_right_sides;
    push @{$values_array[5]}, $num_top_sides;
    push @{$values_array[6]}, $num_bottom_sides;

    if ($depth <= -1) {
      foreach my $y (reverse 1 .. $y_max) {
        foreach my $x (1 .. $x_max) {
          if (defined $xy[$x][$y]) {
            print " $xy[$x][$y]";
          } else {
            print "  ";
          }
        }
        print "\n";
      }
    }
  }

  foreach my $vref (@values_array) {
    print join(',',@$vref),"\n";
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search(array => $vref);
  }
  exit 0;
}

{
  # convex hull centroid
  # HV(d,i) = [ F(i+1); F(i+2) + (d-i)*F(i+1) ]
  #
  my $type;
  $type = 'row';
  $type = 'whole';
  my @values;
  print "$type\n";
  my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');

  foreach my $d (2 .. 10) {
    my $planar = hull_planar($d,$type);
    $planar = planar_bigrat($planar);
    my $centroid = $planar->centroid;
    my $area = $planar->area;
    my ($x,$y) = @$centroid;
    $area = $area->numify;
    $y = $y->numify;
    my $cf = hull_centroid_GR($d);
    print "d=$d $x $y  $cf\n";
    push @values, $x * $area*3;
  }

  shift @values;
  shift @values;
  shift @values;
  print join(',',@values),"\n";
  # # print join(',',map {sprintf '%.2f', $_} @values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;

  sub hull_centroid_GR {
    my ($d) = @_;
    ### hull_centroid_GR(): $d

    {
      # subtracting side triangles

      my $coord = 0;
      my $weight = 0;

      {
        # square  (1/2)*F(d+2)^3
        my $c = F($d+2)/2;
        my $area = F($d+2)**2;
        $coord += $c*$area;
        $weight += $area;
      }

      {
        # top middle corner   (1/6)* F(d+4)*F(d)^2
        my $area = F($d)**2/2;
        my $c = (F($d+1) + F($d+2) + F($d+2)) / 3;
        # $c = F($d+4) / 3;
        $coord -= $c*$area;
        $weight -= $area;
      }

      {
        # 3 to d+1 =
        my $area = ($d+1-3)/2;
        $area = $d/2 - 1;
        my $cp = (0+1+1)/3;
        $cp = 2/3;
        my $cq = (3+3+ $d+1)/3;
        $cq = ($d+7)/3;
        my $c = ($cp + $cq)/2;  # (d+7+2)/3/2
        $c = (1/6)*$d + 3/2;
        my $ca2 = $c * 2*$area;
        $ca2 = ((1/6)*$d + 3/2)*($d/2 - 1)*2;
        $ca2 = ((1/6)*$d + 3/2)*($d - 2);
        $ca2 = (1/6)*$d*$d + (3/2 - 1/3)*$d - 3;
        $ca2 = (1/6)*$d*$d + (7/6)*$d - 3;
        $coord -= $ca2;
        $weight -= 2*$area;
      }

      {
        # 0,0 to 0,3 and 3,0
        # .5  weight 1
        # 1   weight 2
        # 1.5 weight 2
        # c = (1/2 + 2*1 + 2*3/2)/5
        #   = 11/10
        my $area = 5;
        my $c = 11/10;
        my $ca = 11/2;
        $coord -= $ca;
        $weight -= $area;
      }

      # $coord = Math::BigRat->new($weight);
      # $weight = Math::BigRat->new($weight);
      foreach my $i (3 .. $d) {
        ### $i
        my $area = F($i)**2 / 2;

        my $cp = (0 + 0 + F($i)) / 3;
        my $cq = (F($i+1)    # q axis
                  + F($i+2)  # q axis
                  + F($i+1) + ($d-$i+1)*F($i)  # sheared
                 ) / 3;
        my $c = ($cp + $cq)/2;
        $c = (3*F($i+2) + ($d-$i)*F($i)) / 6;
        my $ca2 = $c * 2*$area;
        $ca2 = (3*F($i+2) + ($d-$i)*F($i)) / 6 * 2 * F($i)**2 / 2;
        $ca2 = (3*F($i+2) + ($d-$i)*F($i)) * F($i)**2 / 6;
        $ca2 = (1/2)* F($i)**2 * F($i+2)  +  (1/6)*($d-$i) * F($i)**3;

        $coord -= $ca2;
        $weight -= 2*$area;
      }

      {
        my $total_by_formula = MyRationals::path_tree_depth_to_hull_area($path,$d);
        $weight == MyRationals::path_tree_depth_to_hull_area($path,$d)
          or die "d=$d weight=$weight total=$total_by_formula";
      }
      return $coord / $weight;
    }

    {
      # side triangle
      # centroid 1 + F(i+1) + F(i+2) = F(i+3)+1;
      #          1 + F(i+2) + (d-i)*F(i+1) + F(i+3) + (d-i-1)*F(i+2)
      #   = 1 + F(i+4) + (d-i)*F(i+1) + (d-i)*F(i+2) - F(i+2)
      #   = 1 + F(i+3) + (d-i)*F(i+3)
      # area (F(i+2) - 1)*((d-i-1)*F(i+2) - 1)
      #    - (F(i+1) - 1)*((d-i)*F(i+1) - 1)
      #   =  F(i+2)*(d-i-1)*F(i+2) - F(i+2)
      #           - (d-i-1)*F(i+2) + 1
      #   -  F(i+1)*(d-i)*F(i+1) + F(i+1)
      #           + (d-i)*F(i+1) - 1
      #   =  F(i+2)*(d-i-1)*F(i+2) - (d-i-1)*F(i+2) - F(i)
      #   -  F(i+1)*(d-i)*F(i+1) + (d-i)*F(i+1)
      #   =  F(i+2)*(d-i-1)*F(i+2) - (d-i-1)*F(i+2) - F(i)
      #   -  F(i+1)*(d-i)*F(i+1) + (d-i)*F(i+1)
      #   =
      # i=1 to i=d-1 inclusive

      # middle 1,1 to F(d+1); F(d+2)
      #   1 + F(d+1) + F(d+2) = 1 + F(d+3)
      # middle area (F(d+1)-1)*(F(d+3)-1)/2
      #   = F($d)*(F($d+3)/2 - 1)
      # total
      #   = (1 + F(i+3) + (d-i)*F(i+3)) * (F(2d+3)/2 - d - 1 - ...)
      #     + (1 + F(d+3)) *

      my $coord = 0;
      my $weight = 0;
      # my $coord = Math::BigRat->new(0);
      # my $weight = Math::BigRat->new(0);
      foreach my $i (1 .. $d-1) {
        ### $i
        my ($p1,$q1) = hull_vertex($i,$d);
        my ($p2,$q2) = hull_vertex($i+1,$d);

        $p1 = F($i+1);
        $q1 = F($i+2) + ($d-$i)*F($i+1);
        $p2 = F($i+2);
        $q2 = F($i+3) + ($d-$i-1)*F($i+2);

        my $planar = Math::Geometry::Planar->new;
        $planar->points([[1,1], [$p2,$q2], [$p1,$q1] ]);
        my $centroid = $planar->centroid;
        # my $area = $planar->area;
        # my $area = TriangleArea([[1,1], [$p2,$q2], [$p1,$q1] ]);
        my $area = (($p2-1)*($q1-1) - ($p1-1)*($q2-1))/2;

        $area =  ((F($i+2) - 1)  * (F($i+2) + ($d-$i)*F($i+1) - 1)
                  - (F($i+1) - 1)* (F($i+3) + ($d-$i-1)*F($i+2) - 1)) / 2;

        $area =  (F($i+2) * F($i+2)
                  + F($i+2) * ($d-$i)*F($i+1)
                  - F($i+2)
                  - F($i+2)
                  - ($d-$i)*F($i+1)
                  + 1
                  - F($i+1)* F($i+3)
                  - F($i+1)* ($d-$i-1)*F($i+2)
                  + F($i+1)
                  + F($i+3)
                  + ($d-$i-1)*F($i+2)
                  - 1
                 ) / 2;

        # Cassini's identity
        # F_n^2 - F_(n+1)F_(n-1) = (-1)^(n)
        $area =  (F($i+2) * F($i+2) - F($i+1)* F($i+3)
                  + F($i+2) * F($i+1)
                  + ($d-$i-2)*F($i)
                 ) / 2;

        $area =  (F($i+2) * F($i+1)
                  + ($d-$i-2)*F($i)
                  - (-1)**$i
                 ) / 2;

        my $cp = (1 + $p1 + $p2) / 3;
        my $cq = (1 + $q1 + $q2) / 3;
        my $c = ($cp + $cq)/2;
        $c = (2 + 2*F($i+3) + ($d-$i)*F($i+3)
             ) / 6;

        my $c_weighted = $area * $c;
        $c_weighted =  (F($i+2) * F($i+1)
                        + ($d-$i-2)*F($i)
                        - (-1)**$i
                       )
          * (2 + 2*F($i+3) + ($d-$i)*F($i+3)
            )
            / 12;

        # d'Ocagne's identity
        # F(m)F(n+1) = F(m+1)F(n) + (-1)^n F(m-n)
        # F(m)F(n+1) = F(m+1)F(n) + (-1)^n F(m-n)    m=i+2 n+1=i+1
        # F(i+2)F(i+1) = F(i+3)F(i) + (-1)^n         m=i+2 n+1=i+1


        $c_weighted = (
                       + ($d - $i + 2)                 * F($i+3) * F($i+2) * F($i+1)
                       + ($d*$d - 2*$d*$i + $i*$i - 2) * F($i+3) * F($i)
                       - 2*$i            * F($i)
                       - ($d-$i+2)*(-1)**$i * F($i+3)
                       + 2*($d-2)        * F($i)
                      )
          / 12;

        my $c_weighted_short
          = (
             + ($d - $i + 2)                 * F($i+3) * F($i+2) * F($i+1)
             + ($d*$d - 2*$d*$i + $i*$i - 2) * F($i+3) * F($i)
             + $i*(-1)**$i * F($i+3)
            )
            / 12;

        ### points: [[1,1], [$p1,$q1], [$p2,$q2] ]
        ### $area
        ### $centroid
        ### $cp
        ### $c

        # $coord += 2*$c_weighted;
        $coord += 2*$c_weighted_short;
        $weight += 2*$area;
      }

      # - 2*$i            * F($i)
      my $c_summed = (
                      # sum i=1 to d of - (-1)**($i+3) * F($i+3)
                      + ($d+2) * (-1)**$d * F($d+1)

                      + 2 * F($d)
                      - $d
                      + 2

                     ) / 12;
      $coord += 2*$c_summed;

      my $planar = Math::Geometry::Planar->new;
      $planar->points([[1,1], [F($d+2),F($d+1)], [F($d+1),F($d+2)] ]);
      my $centroid = $planar->centroid;
      # my $area = $planar->area;
      # my $area = TriangleArea([[1,1], [F($d+2),F($d+1)], [F($d+1),F($d+2)] ]);
      my $area = F($d)*(F($d+3)/2 - 1);
      my $c = (1 + F($d+1) + F($d+2)) / 3;
      $c = (F($d+3) + 1) / 3;

      ### points: [[1,1], [F($d+2),F($d+1)], [F($d+1),F($d+2)] ]
      ### $area
      ### $centroid
      ### $c

      $coord += $c * $area;
      $weight += $area;
      ### $coord
      ### $weight

      {
        my $total_by_formula = MyRationals::path_tree_depth_to_hull_area($path,$d);
        $weight == MyRationals::path_tree_depth_to_hull_area($path,$d)
          or die "d=$d weight=$weight total=$total_by_formula";
      }
      return $coord / $weight;
    }
  }

  sub hull_vertex {
    my ($i,$d) = @_;
    return (F($i+1), F($i+2) + ($d-$i)*F($i+1));
  }
}



{
  # (sum F(i)*F(i+1)*F(i+2))/2 = A215037
  # sum i*F(i)*F(i+1)*F(i+2) = 
  my @values;
  foreach my $n (1 .. 20) {
    my $s = sum_nF3($n);
    my $f = 0;
    print "$s $f\n";
    # $s == $f or die;
    push @values, $s;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;

  sub sum_nF3 {
    my ($n) = @_;
    my $sum = 0;
    foreach my $i (1 .. $n) {
      $sum += F($i)*F($i+1)*F($i+2);
    }
    return $sum;
  }
}
{
  # product F(i)*F(i+1)*F(i+2) = A065563
  my @values;
  foreach my $n (1 .. 20) {
    my $s = F($n)*F($n+1)*F($n+2);
    my $f = 0;
    print "$s $f\n";
    # $s == $f or die;
    push @values, $s;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;
}


# {
#   foreach my $d (1 .. 10) {
#     TriangleArea(
#       $planar->points([[1,1], [F($d+2),F($d+1)], [F($d+1),F($d+2)] ]);
#       my $centroid = $planar->centroid;
#       # my $area = $planar->area;
#       # my $area = TriangleArea([[1,1], [F($d+2),F($d+1)], [F($d+1),F($d+2)] ]);
#       my $area = F($d)*(F($d+3)/2 - 1);
#       my $c = (1 + F($d+1) + F($d+2)) / 3;
# 
# }

{
  # centroid centre of gravity -- whole
  # total sum P = 3^d / points 2^d, same Q

  # total area (5*2^n - 4*n - 5)/2
  # X total = a(n+5) = 3 a(n) - 10 a(n+1) + 12 a(n+2) - 6 a(n+3) + a(n+4)
  # x^4-10/3*x^3+4*x^2-2*x+1/3
  # roots 1/3, 1, 1, 1

  my @values;
  foreach my $d (1 .. 13) {
    my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($d);
    # $n_row = 1;
    my $area_total = 0;
    my $x_total = 0;
    my $y_total = 0;
    my $pA = [$d+1, 1];
    my $pB = [1, $d+1];
    my $proot = [1,1];
    foreach my $n ($n_row ... $n_end-1) {
      ### $n
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      # $x1 = to_bigrat($x1);
      # $y1 = to_bigrat($y1);
      # $x2 = to_bigrat($x2);
      # $y2 = to_bigrat($y2);
      my $p1 = [$x1,$y1];
      my $p2 = [$x2,$y2];
      my $area = TriangleArea([$p1, $proot, $p2]);
      $area_total += $area;
      my $x = ($x1+$x2+1)/3;
      my $y = ($y1+$y2+1)/3;
      if ($d <= 2) {
        print "  triangle $x1,$y1 $x2,$y2 1,1  area=$area c=$x,$y\n";
      }
      $x_total += $x * $area;
      $y_total += $y * $area;

      if (0) {  # subtracting to row-only area
        my $p3 = LineIntersection([ $p1, $proot, $pA,$pB ]);
        my $p4 = LineIntersection([ $p2, $proot, $pA,$pB ]);
        $area = - TriangleArea([$p3, $proot, $p4]);
        $area_total += $area;
        $x_total += $x * $area;
        $y_total += $y * $area;
      }
    }
    my $cp = centroid_by_planar($d); # * $area_total * 3;
    my $f = centroid_by_formula($d) * (rationals_area_total_by_formula($d) * 3);
    my $cr = centroid_by_rec($d);
    my $ce = centroid_by_power($d) * 3;
    my $xa = ($x_total / ($area_total||1)) * $area_total * 3;
    $x_total*=3;
    # print "d=$d  $x_total / $area_total     $y_total / $area_total\n";
    print "d=$d  $x_total / $area_total     $xa  $f $cp $cr $ce\n";
    push @values, $x_total*3;
    # push @values, $area_total*2;
  }
  # shift @values;
  # shift @values;
  # shift @values;
  # print join(',', map {sprintf '%.1f', $_} @values),"\n";
  print join(',', @values),"\n";
  # # print join(',',map {sprintf '%.2f', $_} @values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;

  sub QuadArea {
    my ($points) = @_;
    ### $points
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    return $planar->area;
  }

  sub to_bigrat {
    my ($x) = @_;
    require Math::BigRat;
    return Math::BigRat->new($x);
  }

  sub centroid_by_formula {
    my ($d) = @_;

    {
      if ($d == 0) { return 1; }
      return (3 * centroid_by_formula($d-1) * rationals_area_total_by_formula($d-1)
              + $d*$d + $d - 4/3
             )
        / rationals_area_total_by_formula($d);
    }
    {
      if ($d == 0) { return 1; }
      return (centroid_by_formula($d-1) * (3/4 *5*2**$d - 6*$d - 3/2)
              + $d*$d + $d - 4/3
             )
        / (5/2*(2**$d-1) - 2*$d);
    }
    {
      if ($d == 0) { return 1; }
      return (3/2*centroid_by_formula($d-1) * 2*(5/2*(2**($d-1)-1)-2*($d-1))
              + $d*$d + $d - 2
              + (2/3)
             )
        / (5/2*(2**$d-1) - 2*$d);
    }
    {
      # C[d-1], C[d-1]
      # left shear to    C[d-1], 2*C[d-1]
      # right shear to 2*C[d-1],   C[d-1]
      # midpoint  3/2*C[d-1], 3/2*C[d-1]
      # weight 2*R[d-1] = 5*(2^(d-1)-1)-2(d-1)
      #                 = 5*2^(d-1) - 5 - 2d-2
      #                 = 5*2^(d-1) - 2d - 7
      # middle area = 2d-3/2
      # initial triangle area = 1/2 centre 1/3, 1/3
      # middle rectangle area = 2(d-1)  from 1,2 to d,d+1       2,3
      #                                 and  2,1 to d+1,d       3,2
      # centre (1 + d + 2 + d+1)/4 = (2d+4)/4 = d/2+1

      my $prev_area = rationals_area_total_by_formula($d-1);
      my $area_total =  (2*$prev_area
                         + 2*($d-1)
                         + (1/2));
      my $want = rationals_area_total_by_formula($d);
      $area_total == $want or die;

      if ($d == 0) { return 1; }
      return (3/2*centroid_by_formula($d-1) * 2*$prev_area
              + ($d/2+1)    * 2*($d-1)
              + (4/3)       * (1/2)
             )
        / $area_total;
    }
  }

  sub centroid_by_power {
    my ($d) = @_;

    # C[d+1] = (3/2*C[d] *2*R[d] + d^2 + d - 4/3 ) / R[d+1]
    # C[d+1]*R[d+1] = 3*C[d]*R[d] + (d^2 + d - 4/3)
    # G[d+1] = 3*G[d] + d^2 + d - 4/3
    # 3G[d+1] = 3*(3G[d]) + 3*d^2 + 3d - 4
    # 3G[1] = 2
    # 3G[2] = 3*2 + 3*4 + 3*2 - 4 = 20
    # 3G[3] = 3*20 + 3*9 + 3*3 - 4 = 92
    # 3G[4] = 3*92 + 3*16 + 3*4 - 4 = 332
    # A*3   + B    + C   + D = 2
    # A*9   + B*4  + C*2 + D = 20
    # A*27  + B*9  + C*3 + D = 92
    # A*81  + B*16 + C*4 + D = 332
    # matsolve([3,1,1,1; 9,4,2,1; 27,9,3,1; 81,16,4,1], [2;20;92;332])
    # [ 19/4]
    # [ -3/2]
    # [   -6]
    # [-19/4]

    {
      if ($d == 0) { return 1; }
      return ( 19*3**$d - 6*$d*$d - 24*$d - 19 ) / 4
        / 3;
    }
    {
      if ($d == 0) { return 1; }
      return (19/4 * 3**$d
              -3/2 * $d*$d
              -6   * $d
              -19/4
             );
    }
  }

  sub centroid_by_rec {
    my ($d) = @_;
    # 3 a(n) - 10 a(n + 1) + 12 a(n + 2) - 6 a(n + 3) + a(n + 4) == 0
    # x^4 - 6*x^3 + 12*x^2 - 10*x + 3
    # = (x-3)*(x-1)^3
    # so centroid (A*3^k + B ) / R[d]
    if ($d == 0) { return 1; }
    if ($d == 1) { return 2; }
    if ($d == 2) { return 20; }
    if ($d == 3) { return 92; }
    if ($d == 4) { return 332; }
    # if ($d == 5) { return 1082; }
    # if ($d == 6) { return 3368; }
    # if ($d == 7) { return 10268; }
    # if ($d == 8) { return 31016; }
    return (- 3*centroid_by_rec($d-4)
            + 10*centroid_by_rec($d-3)
            - 12*centroid_by_rec($d-2)
            + 6*centroid_by_rec($d-1));
  }
  BEGIN { use Memoize;
          memoize('centroid_by_rec'); }

  sub centroid_by_planar {
    my ($d) = @_;
    my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
    my @points = ([1,1]);
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($d);
    foreach my $n ($n_row .. $n_end) {
      my ($x,$y) = $path->n_to_xy($n);
      push @points, [$x,$y];
    }
    my $planar = Math::Geometry::Planar->new;
    $planar->points(\@points);
    ### @points
    return $planar->centroid->[0];
  }
}

{
  # centroid centre of gravity - row

  my @values;
  foreach my $d (0 .. 6) {
    my $cp = row_centroid_by_planar($d);
    my $f = row_centroid_by_formula($d);
    print "d=$d   cp=$cp $f\n";
    # push @values, $x_total;
    # push @values, $area_total*2;
  }
  shift @values;
  shift @values;
  shift @values;
  # print join(',', map {sprintf '%.1f', $_} @values),"\n";
  print join(',', @values),"\n";
  # # print join(',',map {sprintf '%.2f', $_} @values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;

  sub row_centroid_by_formula {
    my ($d) = @_;

    # C[k+1] = (3/2*C[k] * 2*(5*2^(d-1)-2) + (d+1)*2 ) / (5*2^d-2);
    #        = (C[k] * (15*2^(d-1) - 3*2) + 2*d + 2 ) / (5*2^d-2);
    #        = C[k] * (15*2^(d-1) - 3*2)/(5*2^d-2)   + 2*(d+1)/(5*2^d-2)
    {
      if ($d == 0) { return 1 + 1/3; }
      return (3/2*row_centroid_by_formula($d-1) * 2*(5*2**($d-1)-2)
              + ($d+1)*2 ) / (5*2**$d-2);
    }

    {
      my $prev_area = rationals_area_between_by_formula($d-1);
      my $area_total =  (2*$prev_area
                         + 2);
      my $want = rationals_area_between_by_formula($d);
      $area_total == $want or die;

      if ($d == 0) { return 1 + 1/3; }
      return (3/2*row_centroid_by_formula($d-1) * 2*$prev_area
              + ($d+1)  * 2
             )
        / $area_total;
    }
  }

  sub row_centroid_by_planar {
    my ($d) = @_;
    my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
    my @points;
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($d);
    foreach my $n ($n_row .. $n_end) {
      my ($x,$y) = $path->n_to_xy($n);
      push @points, [$x,$y];
    }
    ($n_row, $n_end) = $path->tree_depth_to_n_range($d+1);
    foreach my $n (reverse $n_row .. $n_end) {
      my ($x,$y) = $path->n_to_xy($n);
      push @points, [$x,$y];
    }
    ### @points
    my $planar = Math::Geometry::Planar->new;
    $planar->points(\@points);
    return $planar->centroid->[0];
  }
}

{
  # centroid of shear
  my $points = [ [6,6], [1,20], [10,1] ];
  $points = [ [1,1], [1,2], [2,1] ];
  {
    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    my $c = $planar->centroid;
    print "$c->[0], $c->[1]\n";
  }

  foreach (1 .. 3) {
    # foreach my $p (@$points) {
    #   $p->[1] += $p->[0];
    # }
    foreach my $p (@$points) {
      $p->[0] += $p->[1];
    }
    print points_str($points),"\n";
    {
      my $planar = Math::Geometry::Planar->new;
      $planar->points($points);
      my $c = $planar->centroid;
      print "$c->[0], $c->[1]\n";
    }
  }
  exit 0;
}

{
  my $p = LineIntersection([ [1,1],[1,5], [5,1],[1,5] ]);
  ### $p
  exit 0;
}

{
  # P,Q mean
  # total sum P = 3^d / points 2^d, same Q
  foreach my $d (0 .. 10) {
    my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($d);
    $n_row = 1;
    my $x_total = 0;
    my $y_total = 0;
    my $count = 0;
    foreach my $n ($n_row ... $n_end) {
      my ($x,$y) = $path->n_to_xy($n);
      $x_total += $x;
      $y_total += $y;
      $count++;
    }
    print "$x_total/$count  $y_total/$count\n";
  }
  exit 0;
}

{
  # BigFloat centroid
  my $planar = Math::Geometry::Planar->new;
  $planar->points([[0,0], [1,0], [1,1], [0,1]]);
  $planar = planar_bigfloat($planar);
  $planar->centroid;
  exit 0;

  # return a copy of $planar which has BigFloat points
  sub planar_bigfloat {
    my ($planar) = @_;
    require Math::BigFloat;
    my $points = $planar->points;
    my $new_points = [ map { [ Math::BigFloat->new($_->[0]),
                               Math::BigFloat->new($_->[1]) ]
                           } @$points ];
    my $new_planar = Math::Geometry::Planar->new;
    $new_planar->points($new_points);
    return $new_planar;
  }
}
{
  # convex hull maximum triangle inside
  #
  #       F(d),F(d+2)
  #               F(d+2),F(d)
  #   d+1,1
  #
  # AB x AC

  my $type;
  $type = 'whole';
  $type = 'row';
  my @values;
  print "$type\n";

  require Math::BigRat;
  foreach my $d (6 .. 80) {
    my $d = Math::BigRat->new($d);
    print is_integer(maxtriangle_by_formula($d)) ? '-' : '*';
  }
  print "\n";

  foreach my $d (2 .. 17) {
    my $planar = hull_planar($d,$type);
    # $planar = planar_bigrat($planar);
    my $triangle = planar_convex_maxtriangle($planar);
    my $area = $triangle->area;
    my $boundary = $triangle->perimeter * 0;
    my $points_str = points_str($triangle->points);
    my $f = maxtriangle_by_formula($d);
    my $diff = ($f == $area ? '' : '****');
    print "d=$d $area $f$diff   $points_str\n";
    push @values, 2*$area;
  }

  shift @values;
  print join(',',@values),"\n";
  # # print join(',',map {sprintf '%.2f', $_} @values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;

  sub maxtriangle_by_formula {
    my ($d) = @_;

    # formula for d >= 6
    # 2 of 6 are 1/2, rest are integers

    # F(2n) = F(n+1)^2 - F(n-1)^2
    #
    # (F(d+2) -(d+1)) * (F(d+2) -1)
    #   - (F(d+2)-1)     * (F(d)-(d+1))/2
    #   - (F(d+2)-(d+1)) * (F(d)-1)    /2
    #   - F(d+1)^2/2
    # 2*F(d+2)^2 - 2*(d+2)*(F(d+2) + 2*(d+1)
    #   - F(d+2)*F(d) - (d+2)*F(d) -       F(d+2) + (d+1)
    #   - F(d+2)*F(d) -       F(d) - (d+2)*F(d+2) + (d+1)
    #   - F(d+1)^2


    my $n = $d+1;
    return (F(2*$n) - ($n+1)*F($n)) / 2;

    return (F(2*$d+2) - ($d+2)*F($d+1)) / 2;

    return (F($d+2)**2 - F($d)**2 - ($d+2) * F($d+1) ) / 2;

    return (2*F($d+2)**2 - 2*($d+2)*F($d+2) + 2*($d+1)
            - F($d+2)*F($d) + ($d+1)*F($d) +       F($d+2) - ($d+1)
            - F($d+2)*F($d) +       F($d) + ($d+1)*F($d+2) - ($d+1)
            - F($d+1)**2) / 2;

    return ((F($d+2) -($d+1)) * (F($d+2) -1)
            - (F($d+2)-1)     * (F($d)-($d+1))/2
            - (F($d+2)-($d+1)) * (F($d)-1)    /2
            - F($d+1)**2/2);
  }

  # $planar is a convex Math::Geometry::Planar.
  # Find the maximum area triangle which fits inside $planar and return that
  # triangle as a new Math::Geometry::Planar.
  sub planar_convex_maxtriangle {
    my ($planar) = @_;
    my $points = $planar->points;
    my $max_area = 0;
    my $max_points = 0;
    foreach my $i (0 .. $#$points-2) {
      foreach my $j ($i .. $#$points-1) {
        foreach my $k ($j .. $#$points) {
          my $triangle_points = [ $points->[$i],
                                  $points->[$j],
                                  $points->[$k] ];
          my $triangle_area = TriangleArea($triangle_points);
          if ($triangle_area > $max_area) {
            $max_points = $triangle_points;
            $max_area = $triangle_area;
          }
        }
      }
    }
    my $triangle = Math::Geometry::Planar->new;
    $triangle->points($max_points);
    return $triangle;
  }
}
{
  # how w,h varies with d,s

  foreach my $d (2 .. 10) {
    my $square_area = minrectfull_square($d);
    print "d=$d  square=$square_area\n";
    foreach my $s (1 .. max(1,$d-2)) {
      my $w = minrectfull_w_from_ds($d,$s);
      my $h = minrectfull_h_from_ds($d,$s);
      my $rect_area = $w*$h;
      printf "%8.1f %8.1f = %8.1f diff %8.1f)\n",
        $w, $h, $rect_area,
          minrectfull_diff_from_ds($d,$s);
    }
    print "\n";
  }
  exit 0;

  sub minrectfull_square {
    my ($d) = @_;
    return (F($d+2) - 1)**2;
  }
  sub minrectfull_w_from_ds {
    my ($d,$s) = @_;
    return minrectfull_wnum_from_ds($d,$s) / sqrt($s*$s+1);
  }
  sub minrectfull_h_from_ds {
    my ($d,$s) = @_;
    return minrectfull_hnum_from_ds($d,$s) / sqrt($s*$s+1);
  }
}

{
  # how diff varies with s

  foreach my $d (2 .. 20) {
    my $prev;
    foreach my $s (1 .. $d-2) {
      my $diff = minrectfull_diff_from_ds($d,$s);
      if (defined $prev) {
        if ($prev == $diff) {
          print " = ";
        } elsif ($prev > $diff) {
          print " > ";
        } else {
          print " < ";
        }
      }
      printf "%.1f", $diff;
      $prev = $diff;
    }
    print "\n";
  }
  exit 0;
}

{
  # diff_incr
  foreach my $d (2 .. 20,
                 # 10,11
                ) {
    print "d=$d\n";

    # w(s)  = (     s*F(d+2) - F(d) + F(d-s+2) ) / sqrt(s^2      + 1)
    # w(s+1)= ( (s+1)*F(d+2) - F(d) + F(d-s+1) ) / sqrt(s^2 + 2s + 2)
    #
    # h(s)^2   = (     s*F(d+2) - F(d) + F(d-s+2) )^2 / (s^2      + 1)
    # h(s+1)^2 = ( (s+1)*F(d+2) - F(d) + F(d-s+1) )^2 / (s^2 + 2s + 2)
    #
    # dw = ((s+1)*F(d+2) - F(d) + F(d-s+1) )^2 * (s^2      + 1)
    #    - (    s*F(d+2) - F(d) + F(d-s+2) )^2 * (s^2 + 2s + 2)

    my $dnum_incr_func = sub {
      my ($s) = @_;

      # d'Ocagne's identity
      # F(m)F(n+1) = F(m+1)F(n) + (-1)^n F(m-n)

      # Cassini's identity
      # F_n^2 - F_(n+1)F_(n-1) = (-1)^(n)

      # negatives
      # F(-n) = - (-1)^n * F(n)

      # F(d) = F($s+1)*F($d-$s) + F($s)*F($d-$s-1)   for s many steps downwards
      return
        (
         + (F($d-1) - ($s-1)*F($d-$s)) * F($d+2)

         # > 0 for d>=2
         + 2*$s * (F($d+2) - 1) - 1

         # >= 0
         +                 F($d)
         + (-1)**($d+$s) * F($s+2)       # s=1 to d-2   s+2=3 to d

         # > 0
         + ($s-1) * F($d-$s)
         +          F($d-$s-2)

        );


      return
        (
         +          F($d+2) * F($d-$s-1)
         -          F($d+1) * F($d-$s)
         +          F($d)

         +          F($d-1)  * F($d+2)
         - ($s-1) * F($d-$s) * F($d+2)


         + (2*$s)     * F($d+2)
         - (2*$s + 1)
         
         # small pos
         + ($s+1) * F($d-$s+2)
         - ($s+2) * F($d-$s+1)

        );

      return
        (
         +          (-1)**($d-$s) * F(2*$s - $d+1) *  F($d-$s)

         + 2*$s * F($s+1) * F($d-$s+2)
         + 2*$s * F($s)   * F($d-$s+1)

         + ($s+1)   * F($d-$s)
         +    F($s+1)*F($s+1)    * F($d-$s)  *    F($d-$s+1)
         +    F($s)  * F($d-$s-1) *    F($d+1)

         - $s                    * F($d-$s) * F($s+1)*F($d-$s+1)
         - ($s-1) * F($s)        * F($d-$s) * F($d-$s-1)

         +    F($s)*F($s+1)      * F($d-$s) * F($d-$s)
         +        2*F($s)        * F($d-$s) * F($d-$s)
         +        2*F($s+1)      * F($d-$s) * F($d-$s)
         - $s * F($s+1)        * F($d-$s) * F($d-$s)
         - $s * F($s)          * F($d-$s) * F($d-$s)


         +          (-1)**$d     * F(-$s-1)
         +         (-1)**($d-$s) * F($s)
         - (2*$s + 1)
         - (-1)**($d+1)     # +/- 1

         +          F($d)
         -          F($d-$s+1)
        );

      return
        (
         - $s     * F($d-$s+2) * F($d+2)
         + ($s+1) * F($d-$s+1) * F($d+2)

         -          F($d-$s+2) * F($d+1)
         +          F($d-$s+1) * F($d+1)

         - ($s+2) * F($d-$s+1)
         + ($s+1) * F($d-$s+2)

         +              F($d+2) * F($d+1)
         -              F($d+2) * F($d)
         + (2*$s)     * F($d+2)
         +              F($d)
         - (2*$s + 1)
        );

      return
        (+ ($s+1)*F($d+2)*($s+1)*F($d+2)
         - F($d)*($s+1)*F($d+2)
         + F($d-$s+1)*($s+1)*F($d+2)

         + ($s+1)*F($d+2)*F($d+1)
         - F($d)*F($d+1)
         + F($d-$s+1)*F($d+1)

         - ($s+1)*F($d+2)*($s+2)
         + F($d)*($s+2)
         - F($d-$s+1)*($s+2)

         - $s*F($d+2)*$s*F($d+2)
         + F($d)*$s*F($d+2)
         - F($d-$s+2)*$s*F($d+2)

         - $s*F($d+2)*F($d+1)
         + F($d)*F($d+1)
         - F($d-$s+2)*F($d+1)

         + $s*F($d+2)*($s+1)
         - F($d)*($s+1)
         + F($d-$s+2)*($s+1)

         - (F($d+2) - 1)**2 * (2*$s + 1)
        );

      return
        (($s+1)*F($d+2)*($s+1)*F($d+2) - F($d)*($s+1)*F($d+2) + F($d-$s+1)*($s+1)*F($d+2))
          + (($s+1)*F($d+2)*F($d+1) - F($d)*F($d+1) + F($d-$s+1)*F($d+1))
            - (($s+1)*F($d+2)*($s+2) - F($d)*($s+2) + F($d-$s+1)*($s+2))

              - (     $s*F($d+2)*$s*F($d+2) - F($d)*$s*F($d+2) + F($d-$s+2)*$s*F($d+2) )
                - (     $s*F($d+2)*F($d+1) - F($d)*F($d+1) + F($d-$s+2)*F($d+1) )
                  + (     $s*F($d+2)*($s+1) - F($d)*($s+1) + F($d-$s+2)*($s+1) )

                    - (F($d+2) - 1)**2 * (2*$s + 1)
                      ;

      return (($s+1)*F($d+2) - F($d) + F($d-$s+1)) * (($s+1)*F($d+2) - F($d) + F($d-$s+1) )
        - (F($d+2) - 1)**2 * ($s**2 + 2*$s + 2)
          -
            ( (     $s*F($d+2) - F($d) + F($d-$s+2) )*( $s*F($d+2) + F($d+1) - ($s+1) )
              - (F($d+2) - 1)**2 * ($s**2 + 1) );
    };

    my $any_star;
    foreach my $s (1 .. $d-2) {
      my $dnum = minrectfull_dnum_from_ds($d,$s);
      my $dnum_incr = minrectfull_dnum_incr_from_ds($d,$s);
      my $f = $dnum_incr_func->($s);
      my $star = ($f == $dnum_incr ? '' : '****');
      $any_star ||= $star;
      printf "s=%2d dnum %5d dnum_incr %5d %5d$star\n",
        $s, $dnum, $dnum_incr, $f;
    }
    if ($any_star) {
      print "bad\n";
    }
    # foreach my $s (1 .. $d-2) {
    #   my $dh = minrectfull_hum_incr_from_ds($d,$s);
    #   print "s=$s dh $dh\n";
    # }
  }
  exit 0;

  sub minrectfull_diff_from_ds {
    my ($d,$s) = @_;
    return ( minrectfull_wnum_from_ds($d,$s)
             * minrectfull_hnum_from_ds($d,$s) / ($s**2 + 1)
             - (F($d+2)-1)**2);
    return minrectfull_dnum_from_ds($d,$s) / ($s*$s + 1);
  }
  sub minrectfull_dnum_from_ds {
    my ($d,$s) = @_;
    return ( minrectfull_wnum_from_ds($d,$s) * minrectfull_hnum_from_ds($d,$s)
             - (F($d+2)-1)**2 * ($s**2 + 1));
  }
  sub minrectfull_dnum_incr_from_ds {
    my ($d,$s) = @_;
    return (minrectfull_dnum_from_ds ($d,$s+1)
            - minrectfull_dnum_from_ds ($d,$s));
  }
  sub minrectfull_wnum_incr_from_ds {
    my ($d,$s) = @_;
    return (minrectfull_wnum_from_ds  ($d,$s+1)**2 * ($s**2 + 1)
            - minrectfull_wnum_from_ds($d,$s  )**2 * (($s+1)**2 + 1));
  }
  sub minrectfull_hum_incr_from_ds {
    my ($d,$s) = @_;
    return (minrectfull_hnum_from_ds  ($d,$s+1)**2 * ($s**2 + 1)
            - minrectfull_hnum_from_ds($d,$s  )**2 * (($s+1)**2 + 1));
  }
}
{
  # ratio minrect / hull area
  #
  # minrect = AMR(d) = (F(d+2) - 1)^2
  # hull    = H(d)   = F(2*d+3)/2 - d - 1
  #
  # ratio = phi^(2d+4)/5 / (phi^(2d+3)/sqrt(5)/2)
  #       = phi * 2*sqrt(5)/5
  #       = phi * 2/sqrt(5)
  #       = (1+sqrt(5))/2 * 2/sqrt(5)
  #       = (1+sqrt(5)) / sqrt(5)
  #       = 1 + 1/sqrt(5)
  #
  # F(n) = ((phi^n - (-phi)^-n) / sqrt(5)
  # F(n) -> phi^n / sqrt(5)

  my $phi = (1 + sqrt(5)) / 2;
  foreach my $d (1 .. 20) {
    my $planar = hull_planar ($d);
    my $hull_area = $planar->area;
    my $log_hull_area = log($hull_area) / log($phi);
    my $rect = $planar->minrectangle;
    my $rect_area = $rect->area;
    my $log_rect_area = log($rect_area) / log($phi);
    my $f = $rect_area / $hull_area;
    # printf "%.4f  %.2f %.2f\n",
    #   $f, $log_rect_area, $log_hull_area;
    printf "%.1f %.1f    %.1f %.1f\n",
      $rect_area,
        ($phi ** ($d+2) / sqrt(5))**2,
          $hull_area,
            $phi ** (2*$d+3) / sqrt(5) / 2;
  }
  exit 0;
}

{
  # minimum rectangle on each side

  foreach my $d (8) {
    my $minside = F($d+2)-1;
    my $minarea = $minside**2;
    print "d=$d  side=$minside  area=$minarea\n";
    my $points = hull_points_aref($d,'whole');

    foreach my $i (0 .. $#$points) {
      my $p1 = $points->[$i];
      my $p2 = $points->[$i-1];
      my ($x1,$y1) = @$p1;
      my ($x2,$y2) = @$p2;
      my $dx = $x2 - $x1;
      my $dy = $y2 - $y1;
      if ($x1 < $y1 && $dx > 0 && $dy >= $dx) {
        my $s = $dy / $dx;
        ### $s
        my $i = $d-$s;
        my $x2f = F($i+1);
        my $y2f = F($i+2) + ($d-$i)*F($i+1);
        print "  % side $x1,$y1 to $x2,$y2 dx=$dx dy=$dy slope $s  x2f,y2f=$x2f,$y2f\n";

        my $origin = [1,1];
        my $pup = [F($d+1),F($d+2)];
        my $pright = [F($d+2),F($d)];
        my $corner_west = PerpendicularFoot([$p1,$p2,  $origin]);
        my $corner_north = PerpendicularFoot([$p1,$p2, $pup]);

        my $corner_south = PerpendicularFoot([$corner_west,$origin,
                                              $pright]);
        my $corner_east = PerpendicularFoot([$corner_north,$pup,
                                             $pright]);
        ### $corner_west
        ### $corner_north
        ### $corner_south
        ### $corner_east
        my $rect = Math::Geometry::Planar->new;
        $rect->points([ $corner_west,
                        $corner_north,
                        $corner_east,
                        $corner_south ]);

        my $want_area = $rect->area;
        my $b1 = $rect->perimeter;
        my $w1 = SegmentLength([$corner_west,$corner_south]);
        my $h1 = SegmentLength([$corner_west,$corner_north]);

        printf "  (%.2f,%.2f)--(%.2f,%.2f)--(%.2f,%.2f)--(%.2f,%.2f)\n",
          @$corner_west,
            @$corner_north,
              @$corner_east,
                @$corner_south;

        my $small = 1/sqrt($s**2 + 1);
        my $big = $s/sqrt($s**2 + 1);
        my $width = (- $p1->[0] * $big + $p1->[1] * $small)
          - (- $pright->[0] * $big + $pright->[1] * $small);
        my $height = ($pup->[0] * $small + $pup->[1] * $big)
          - ($origin->[0] * $small + $origin->[1] * $big);

        # # s = d-i  i=d-s
        # $width = (F($i+1) + $s*F($d) - F($d+2)) / sqrt($s**2 + 1); wrong
        # $height = (F($d) + $s*F($d+2) - ($s+1)) / sqrt($s**2 + 1);
        #
        $width = ( minrectfull_wnum_from_ds($d,$s)

                   #   F($d+2) - F($i+1))*$s
                   # + (F($i+2) + ($d-$i)*F($i+1)) - F($d)

                   # (F($d+2)-1) * $s
                   # - F($d)         # negative
                   # + F($i+2)
                   # + $s
                 )
          / sqrt($s**2 + 1);
        $height = (
                   minrectfull_hnum_from_ds($d,$s)
                   # (F($d+2)-1)*$s
                   # + F($d+1)       # positive
                   # - 1
                  )
          / sqrt($s**2 + 1);
        # my $wh = ($w*$h + 2*(F($d+2)-1)**2) / ($s**2 + 1);

        my $dnum =
          + ($s**2 - $s - 1) * F($d+2)**2
            + (2*$s - 1) * F($d+2) * F($d+1)
              + F($d+1) * F($d+1)

                + $s * F($i+2) * F($d+2)
                  - ($s*$s - 3) * F($d+2)

                    + F($i+2) * F($d+1)
                      - ($s+1) * F($d+1)

                        - ($s+1)*F($i+2)

                          -1;

        $dnum =
          (
           (F($d+2)-1) * ($d-$i)
           - F($d)
           + F($i+2)
           +  ($d-$i)
          ) * (
               (F($d+2)-1)* ($d-$i)
               + F($d+1)
               - 1
              )
            - (F($d+2)-1)*(F($d+2)-1) * $s**2
              - (F($d+2)-1)*(F($d+2)-1);

        # 1 <= s=d-i  <= d-2
        $dnum =
          (F($d+2)-1) * (
                         + ($s-1) * F($d+1)
                         - ($s+2) * F($d)

                         + $s * F($i+2)
                         + $s*$s - $s + 1
                        )
            + F($d)*F($d)

              + $s*(F($d+2)-1)
                - $s*F($d)

                  + (F($d+2)-1) * F($i+2)
                    - F($d)     * F($i+2)
                      ;

        # d-2 <= s <= 1
        # 2 <= i <= d-1
        # d-2 >= s >= 1
        $dnum =
          (
           + $s   * F($d+1) *F($d+1)   # zero when s=1
           -        F($d+1) *F($d+1)

           +      F(-$s)   * F($d+1) * F($d+1)
           +      F(-$s+1) * F($d+2) * F($d+1)

           + $s * F(-$s+1) * F($d+2) *F($d+2)
           + $s * F(-$s)   * F($d+1) *F($d+2)
           - $s            * F($d)   *F($d)
           -                 F($d)   *F($d)
           - 3             * F($d)   *F($d+1)

           + $s*$s *F($d)
           + 3     *F($d)
           - $s*$s


           + $s*$s *F($d+1)           # zero when s=1
           - $s    *F($d+1)

           +      F($d+1)              # zero when s=1
           -      F($d-$s+2)

           +      F($d+1)              # zero when s=1
           - $s * F($d-$s+2)

           - 1
          );

        # + F($s-1 + 1) * F($d-($s-1))
        # + F($s-1    ) * F($d-($s-1)-1)
        # = F($d)
        # s=1 F(s)=1 F(s+1)=1

        # F(d) = F($s+1)*F($d-$s) + F($s)*F($d-$s-1)   for s many steps downwards

        # d'Ocagne's identity
        # F(m)F(n+1) = F(m+1)F(n) + (-1)^n F(m-n)

        $dnum =
          (
           + ($s**2 + 3) * F($s)   * F($d-$s+1)
           +               F($s)   * F($d-$s)   * F($d-$s+2)

           + ((-2)*F($s)  + $s)  * F($s+1)   * (-1)**($d-$s)
           +       F($s)*F(-$s)        * (-1)**$d

           + (($s-2)*F($s) - ($s+1)*F($s+1) + 2*$s)       * F($d-$s)   * F($d+2)
           - (($s)*F($s)        + F($s+1) - $s) * F($s)   * F($d-$s+1) * F($d-$s-1)
           +                                    F($s+1)   * F($d-$s+1) * F($d-$s-1)

           - (($s)*F($s)        + F($s+1) - $s - 2) * F($s+1) * F($d-$s+1) * F($d-$s)


           - ($s**2 + 1)

           # zeros when s=1
           + ($s-1) * F($s+1) * F($d-$s)   * F($d+1)
           + ($s-1) * F($s+1) * F($d-$s+1) * F($d+1)

           + ($s+1)           * F($s+1) * F($d-$s)
           - ($s+1)           * F($s)   * F($d-$s)
           + ($s**2 - $s + 2) * F($s+1) * F($d-$s+2)
           - ($s+1)                     * F($d-$s+2)
          );

        my $want_dnum = $want_area * ($s**2 + 1) - (F($d+2)-1)**2 * ($s**2 + 1);

        # (F(d+2)-1)^2*(s^2+1)
        # <= (F(d+2)-1)*s * (F(d+2)-1)*(s+1)
        my $awh = $width * $height;
        print "  w1=$w1 h1=$h1  area = $want_area  b1=$b1\n";
        print "  w=$width h=$height  awh = $awh\n"; #  cf wh=$wh = $w*$h\n";
        print "  want_dnum=$want_dnum  dnum=$dnum\n";
      }
    }
  }
  exit 0;
}

{
  # convex hull minimum rectangle
  # whole, square
  #   A188516 area       n=1 value=4 is (F(n+3)-1)^2
  #   A204644 perimeter  n=1 value=8 is 4*(F(n+3) - 1)
  #   for(n=1,n=10, print1((F(n+3)-1)^2,", "))
  #   for(n=1,n=10, print1((F(n+3)-1)*4,", "))
  #   
  #
  #   hull = F[2d+3]/2-d-1
  #   minrect = (F[d+2] - 1)^2
  #   minrect/convex   min 1 when convex=square, max 2 when convex=triangle
  #   hr(d) = (F(2*d+3)/2-d-1) / (F(d+2)-1)^2 * 1.0
  #   for(d=1,d=8, print1(hr(d),", "))
  #   -> .6909830056250525758977579079
  #      .6909830056250525758977383035
  #      .6909830056250525
  #   = (3-phi)/2
  #
  #   g(x) = x/(1-x-x^2)
  #   (g(x)-1)^2 = (x/(1-x-x^2) - 1)^2
  #              = ( (x -(1-x-x^2)) / (1-x-x^2) )^2
  #              = ( (x - 1 + x + x^2)) / (1-x-x^2) )^2
  #              = ( (-1 + 2x + x^2)) / (1-x-x^2) )^2
  #   for(d=1,d=20, print1((F(d)-1)^2,","))
  #
  # row
  #   rect point Nrow+1 to F(d+1)+F(d+2)
  #   area = 2*F(d) * (F(d+3) - d - 2);
  #
  #
  #     *---------------*     X+Y = F(d+1)+F(d+1) = F(d+3)
  #     |               |
  #     |               |
  #     |               |
  #     *---------------*     X+Y = d+1
  #
  #   X-Y = 2d+1 - 1 = 2d
  #
  #     area = (F(d+3) - d - 1) * 2*d


  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
  my $type;
  $type = 'whole';
  $type = 'row';
  my @values;
  my $prev_log = 0;
  print "$type\n";
  foreach my $depth (2 .. 12) {
    my $planar = hull_planar($depth,$type);
    foreach my $p (@{$planar->points}) {
      ($p->[0],$p->[1]) = ($p->[0] - $p->[1], # rotate +45
                           $p->[0] + $p->[1])
    }
    my $minrect = $planar->minrectangle;
    my $area = $minrect->area;
    my $perimeter = $minrect->perimeter;

    my $f;
    my $s_max = F($depth+1);
    my $s_min = $depth + 2;
    my $d_max = F($depth);
    my $d_min = - $d_max;
    my $x_min = ($s_min + $d_min) / 2;
    my $x_max = ($s_max + $d_max) / 2;
    my $y_min = ($s_min - $d_min) / 2;
    my $y_max = ($s_max - $d_max) / 2;
    if ($type eq 'row') {
      $f = ($x_max - $x_min) * ($y_max - $y_min);
      $f = (2*F($depth+3) - 2*$depth - 4);
      $f = (2*F($depth+1)) * (F($depth+3) - $depth - 2);
    } else {
      $f = 0; # F($depth+3) + $depth-2;
    }
    my $diff = ($f == $area ? '' : '***');
    print "depth=$depth  a=$area p=$perimeter  formula=$f$diff\n";
    push @values, $area;
    # push @values, $perimeter;

    print "  minr  ";
    $minrect->scale(2);
    foreach my $p (@{$minrect->points}) {
      if (planar_isboundary($minrect, $p)) {
        print "  $p->[0],$p->[1]";
      }
    }

    print "  bound ";
    foreach my $p (@{$planar->points}) {
      if (planar_isboundary($minrect, $p)) {
        print "  $p->[0],$p->[1]";
      }
    }
    print "\n";
    print "  bsumd ";
    foreach my $p (@{$planar->points}) {
      if (planar_isboundary($minrect, $p)) {
        my $sum = $p->[0] + $p->[1];
        my $diff = $p->[0] - $p->[1];
        print "  $sum,$diff";
      }
    }
    print "\n";

    print "  s $s_min $s_max d $d_min $d_max  x $x_min $x_max y $y_min $y_max\n";

    # print "  N     ";
    # foreach my $p (@{$planar->points}) {
    #   my $n = $path->xy_to_n($p->[0],$p->[1]);
    #   printf " %b", $n;
    # }
    # print "\n";
  }

  shift @values;
  print join(',',@values),"\n";
  # print join(',',map {sprintf '%.2f', $_} @values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;
}



{
  # diff
  my $any_neg;
  my $any_star;
  foreach my $d (2 .. 20,
                 # 10,11
                ) {

    my $dnum_func = sub {
      my ($s) = @_;

      # d'Ocagne's identity
      # F(m)F(n+1) = F(m+1)F(n) + (-1)^n F(m-n)

      # Cassini's identity
      # F_n^2 - F_(n+1)F_(n-1) = (-1)^(n)

      # negatives
      # F(-n) = - (-1)^n * F(n)

      # F(d) = F($s+1)*F($d-$s) + F($s)*F($d-$s-1)   for s many steps downwards

      return
        (
         + ($s-1)* F($d+1)*F($d-1)
         - 2*   F($d) * F($d)
         +          F($d)
         +          F($d-2)
         
         +     F($d-$s+2) *F($d+1)
         + $s     * F($d-$s+2) *F($d+1)
         + $s     * F($d-$s+1) *F($d)
         + $s     *F($d)       *F($d-$s)
         + $s     * F($d)      *F($d-1)
         - 1      * F($d)      *F($d-1)
         - 3      * F($d)      *F($d+1)
         
         + $s*$s  * F($d)
         - $s*$s
         - 1
         
         
         + 2      * F($d+1)
         - $s     * F($d-$s+2)
         
         + $s*$s  * F($d+1)
         - $s     * F($d+1)
         
         +          F($d+1)
         -          F($d-$s+2)
        );

      return
        (
         + $s* F($d+2)*F($d-1)
         +     F($d-$s+2) *F($d+1)
         -     F($d+2)*F($d+2)

         + $s * F($d-$s+2) *F($d+2)
         -     F($d) * F($d+1)

         + 2      * F($d)
         -          F($d-1)
         + $s*$s  * F($d)
         - $s*$s
         - 1


         + 2      * F($d+1)
         - $s     * F($d-$s+2)

         + $s*$s  * F($d+1)
         - $s     * F($d+1)

         +          F($d+1)
         -          F($d-$s+2)
        );

      return
        (
         - $s*  F($d)*F($d+2)
         + $s * F($d-$s+2)*F($d+2)
         + F($d-$s+2)*F($d+1)

         + $s* F($d+2)*F($d+1)
         -     F($d) * F($d+1)

         + F($d)*($s+1)
         - F($d-$s+2)*($s+1)

         - ($s*$s+$s) * F($d+2)
         + 2*$s*$s    * F($d+2)
         - F($d+2)*F($d+2)
         + 2*F($d+2)
         - $s*$s
         - 1
        );
    };

    foreach my $s (1 .. $d-2) {
      my $dnum = minrectfull_dnum_from_ds($d,$s);
      my $f = $dnum_func->($s);
      my $star = ($f == $dnum ? '' : '****');
      $any_star ||= $star;
      $any_neg ||= ($f < 0);
      printf "s=%2d dnum %5d  %5d$star\n",
        $s, $dnum, $f;
    }
  }
  if ($any_star) {
    print "bad\n";
  }
  if ($any_neg) {
    print "negatives\n";
  }
  exit 0;
}



{
  my $d = 8;
  sub minrectfull_wnum_from_ds {
    my ($d,$s) = @_;
    return ( F($d+2) * $s
             - F($d)
             + F($d-$s+2));
    return ( (F($d+2)-1) * $s
             - F($d)
             + F($d-$s+2)
             + $s);
  }
  sub minrectfull_hnum_from_ds {
    my ($d,$s) = @_;
    return ( (F($d+2)-1)* $s
             + F($d+1)
             - 1);
  }
  my $dw = sub {
    my ($s) = @_;
    return (
            + F($d+2)
            - F($d-$s)
           );
  };
  my $dh = sub {
    my ($s) = @_;
    return (
            F($d+2)-1
           );
  };
  foreach my $s (1 .. $d-2) {
    print "s=$s w ", minrectfull_wnum_from_ds($d,$s+1) - minrectfull_wnum_from_ds($d,$s), "  ", $dw->($s), "\n";
  }
  foreach my $s (1 .. $d-2) {
    print "s=$s h ", minrectfull_hnum_from_ds($d,$s+1) - minrectfull_hnum_from_ds($d,$s), "  ", $dh->($s), "\n";
  }
  print minrectfull_dnum_from_ds($d,1),"\n";

  # W(1) = F($d+2)-1 - F($d) + F($d+1) + 1
  #      = F($d+2) - F($d) + F($d+1)
  #      = 2*F($d+1)
  # H(1) = F($d+2)-1 + F($d+1) - 1
  #      = F($d+3) - 2
  # diff(1)
  #  = 2*F($d+1)*( F($d+3) - 2 ) - (F($d+2)-1)**2 * 2
  #  = 2*F($d+1)*F($d+3) - 4*F($d+1) - 2*F($d+2)*F($d+2) + 2*2*F($d+2) - 2
  #  = 2*F($d+1)*F($d+3) - 4*F($d+1) - 2*F($d+2)*F($d+2) + 2*2*F($d+2) - 2
  #  = 2*F($d+1)*F($d+3) - 4*F($d+1) - F($d+2)*F($d+2) + 2*F($d+2) - 1
  print "W(1) ",minrectfull_wnum_from_ds($d,1),"  ",2*F($d+1),"\n";
  print "H(1) ",minrectfull_hnum_from_ds($d,1),"  ",F($d+3) - 2,"\n";
  say minrectfull_dnum_from_ds($d,1);
  say 2*F($d+1)*F($d+3) - 4*F($d+1) - 2*F($d+2)*F($d+2) + 2*2*F($d+2) - 2;
  say 2*F($d+1)*F($d+3) - 2*F($d+2)*F($d+2) - 4*F($d+1) + 4*F($d+2) - 2;
  say 2*F($d+1)*F($d+3) - 2*(F($d+3) - F($d+1))**2 + 4*F($d) - 2;
  say 2*F($d+1)*F($d+3) - 2*(F($d+3)**2 - 2*F($d+3)*F($d+1) + F($d+1)**2) + 4*F($d) - 2;
  say 2*F($d+1)*F($d+3) - 2*F($d+3)**2 + 4*F($d+3)*F($d+1) - 2*F($d+1)**2 + 4*F($d) - 2;
  say - 2*F($d+3)**2 + 6*F($d+3)*F($d+1) - 2*F($d+1)**2 + 4*F($d) - 2;
  say - 2*F($d+3)**2 + 6*F($d+3)*F($d+1) - 2*(F($d+3)-F($d+2))**2 + 4*F($d) - 2;
  say - 2*F($d+3)**2 + 6*F($d+3)*F($d+1) - 2*(F($d+3)**2 - 2*F($d+3)*F($d+2) + F($d+2)**2) + 4*F($d) - 2;
  say - 2*F($d+3)**2 + 6*F($d+3)*F($d+1) - 2*F($d+3)**2 + 4*F($d+3)*F($d+2) - 2*F($d+2)**2 + 4*F($d) - 2;
  say - 4*F($d+3)**2 + 6*F($d+3)*F($d+1) + 4*F($d+3)*F($d+2) - 2*F($d+2)**2 + 4*F($d) - 2;
  say - 4*F($d+3)**2 + 2*F($d+3)*F($d+1) + 4*F($d+3)*F($d+3) - 2*F($d+2)**2 + 4*F($d) - 2;
  say 2*F($d+3)*F($d+1) - 2*F($d+2)**2 + 4*F($d) - 2;
  say 2*F($d+3)*F($d+1) - 2*F($d+2)**2 + 4*F($d) - 2;
  # n = d+2
  # r = 1
  say 4*F($d) - 2 - 2 * (-1)**($d+2-1) * F(1)**2;
  say 4*F($d) - 2 - 2 * (-1)**($d+2-1);

  exit 0;

  # Catalan's identity
  # F_n^2 - F_(n+r)F_(n-r) = (-1)^(n-r) F_r^2
}



{
  my $d = 8;
  my $s = 1;
  my $i = $d-$s;

  my $total = 0;
  my @sorted;
  foreach my $value (
                     + $i       * F($d)      # equal
                     + F($d-$i) * F($i+1)
                     - $d       * F($d),

                     + F($i+2)      * (F($d+1)-1)
                     - F($d)   *(F($d+2)-1)
                     + F($d+1),

                     - $i*$d*2      *(F($d+2)-1)
                     + $i*$i        *(F($d+2)-1)
                     + $d*$d *(F($d+2)-1),


                     + $d*F($i+2)   *(F($d+2)-1),
                     - $i*F($i+2)   *(F($d+2)-1),
                     - $i*F($d-1)   *(F($d+2)-1),
                     - F($d+2)      *(F($d+2)-1),
                     + $d*F($d-1)   *(F($d+2)-1),
                     + F($d) * F($d),

                     - 1,

                     + F($d-$i-1) * F($i),

                    ) {
    print "$value\n";
    $total += $value;
    push @sorted, $value;
  }
  @sorted = sort {$a<=>$b} @sorted;
  print "total $total\n";
  $, = ' ';
  print "sorted @sorted\n";
  exit 0;
}


{
  # smallest diff is at i=d-1 which is slope s=1 top-left diagonal

  #  decr    incr     incr
  #  (d+1-i)*F(i+2) - i*F(d-1)

  # (d+1-i)*F(i+2) - i*F(d-1) - ( (d+1-(i+1))*F(i+3) - (i+1)*F(d-1) )
  # = (d+1-i)*F(i+2) - i*F(d-1) - (d+1-(i+1))*F(i+3) + (i+1)*F(d-1)
  # = (d+1-i)*F(i+2) - (d+1-(i+1))*F(i+3) + (i+1)*F(d-1) - i*F(d-1)
  # = (d+1-i)*F(i+2) - (d+1-(i+1))*F(i+3) + F(d-1)
  # = (d+1-i)*F(i+2) - (d+1-i - 1)*F(i+3) + F(d-1)
  # = (d+1-i)*F(i+2) - (d+1-i)*F(i+3) + F(i+3) + F(d-1)
  # = (d+1-i)*(F(i+2) - F(i+3)) + F(i+3) + F(d-1)
  #
  # F(d) = F(i+1)*F(d-i) + F(i)*F(d-i-1)
  #
  # = (d+1-i)*(F(i+2) - F(i+3)) + F(i+3) + F(i+1)*F(d-i) + F(i)*F(d-i-1)
  # = (i-d-1)*F(i+1) + F(i+3) + F(i+1)*F(d-i) + F(i)*F(d-i-1)



  foreach my $d (1 .. 8) {
    print "d=$d ";
    my $dpart =
      (
       + $d*F($d-1)     *(F($d+2)-1)
       + $d*$d *(F($d+2)-1)
       + F($d)
       + F($d)   * F($d)
       - F($d+2) *(F($d+2)-1)
       - F($d)   *(F($d+2)-1)
       - $d  * F($d)
       + F($d+1)

      );

    my $prev;
    foreach my $i (1 .. $d-1) {
      my $ipart = (
                   - (
                   # as i increases becomes more positive
                      + $i             *F($i+2)
                      + $i * F($d-$i-2)*F($i+2)
                      - ($d+1)         *F($i+2)

                      + $i * F($d-$i-1) *F($i+1)
                      - $i * F($d-$i-2) *F($i+1)
                     ) *(F($d+2)-1)


                   # - F($i+2) * F($d)
                   # + $i      * F($d)
                   # - 1
                   # 
                   # + $i*$i        *(F($d+2)-1)     # i^2 >= i
                   # - $i*$d*2      *(F($d+2)-1)
                  );
      my $value = $ipart + $dpart;
      my $relation = (! defined $prev ? ''
                      : $prev > $value ? '>'
                      : $prev < $value ? '<'
                      : '=');
      print " $relation $value";
      if (defined $prev && $value > $prev) {
        print "\n";
        die "oops prev=$prev value=$value\n";
      }
      $prev = $value;
    }
    print "\n";
  }
  exit 0;
}

{
  foreach my $d (1 .. 30) {
    foreach my $s (1 .. $d-2) {
      my $i = $d - $s;
      my $pos =    + F($d+1)       # positive
        - 1;
      my $neg =
        - F($d)         # negative
          + F($i+2)
            + $s
              ;

      $pos =
        $s*F($d-1)    *(F($d+2)-1)
          + $s*F($i+2)  *(F($d+2)-1)
            + $s*$s     *(F($d+2)-1)
              + F($i+2) * F($d+1)
                + F($d)
                  ;
      $neg =
        - F($d+2) * (F($d+2)-1)
          - F($d) * (F($d+2)-1)
            ;


      # my $pos =
      #   + ($s-1) * F($d+1)
      #     + $s*(F($d+2)-1)
      #       # + $s * F($i+2)
      #       #   + $s*$s - $s + 1
      #       #     + $s
      #       ;
      # my $neg = - ($s+2) * F($d);

      # $pos =
      #   + (F($d+2)-1) * ($s*$s - $s + 1)
      #
      #   + F($d+2) * ($s-1) * F($d+1)
      #     - F($d+2) * ($s+2) * F($d)
      #       + F($d)*F($d)
      #
      #         - ($s-1) * F($d+1)
      #           + ($s+2) * F($d)
      #
      #               - $s*F($d)
      #
      #                 + (F($d+2)-1) * $s * F($i+2)
      #                   + (F($d+2)-1) * F($i+2)
      #                     - F($d)     * F($i+2)
      #                       ;
      # $neg = 0;



      my $dnum = ($pos+$neg < 0 ? '****' : '');
      print "pos $pos neg $neg$dnum\n";
    }
    print "\n";
  }
  exit 0;
}




{
  # Planar minrectangle() on colinear points
  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  $planar->points([ [0,0], [1,0], [2,0], [3,0], [4,0] ]);
  my $p2 = $planar->minrectangle;
  # my $p2 = $planar->bbox;
  print points_str($planar->points),"\n";
  print points_str($p2->points),"\n";
  exit 0;
}

{
  # convex hull points on boundary
  #
  # of whole is 1,3,5,8,... is 1 then F(depth+3)
  # A071679 Least k such that the maximum number of elements among the continued fractions for k/1, k/2, k/3, k/4 ...., k/k equals n.
  # A071679 ,1,3,5,8,13,21,34,55,89,144,233,377,610,987,1597,2584,4181,

  # of one row is F(depth+3) + d-2

  my $type;
  $type = 'whole';
  $type = 'row';
  my @values;
  my $prev_log = 0;
  print "$type\n";
  foreach my $depth (0 .. 10) {
    my $planar = hull_planar($depth,$type);
    my $points = $planar->points;
    my $count = planar_count_boundary_integers($planar);
    my $f;
    if ($type eq 'row') {
      $f = F($depth+3) + $depth-2;  # row
    } else {
      $f = F($depth+3) + 2*$depth-2;  # whole
    }
    my $diff = ($f == $count ? '' : '***');
    print "depth=$depth  $count  formula=$f$diff\n";
    if (@$points < 10) {
      print "  points ",points_str($points),"\n";
    }
    push @values, $count;
  }

  shift @values;
  print join(',',@values),"\n";
  # print join(',',map {sprintf '%.2f', $_} @values),"\n";
  Math::OEIS::Grep->search(array => \@values);
  exit 0;
}

{
  # HCS turn left,right
  require Math::NumSeq::PlanePathTurn;
  require Math::BaseCnv;
  require Math::PlanePath::GrayCode;
  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'Bird');
  my $seq = Math::NumSeq::PlanePathTurn->new (planepath_object => $path,
                                              turn_type => 'Right');
  foreach my $n ($path->n_start+1 .. 255) {
    # if (($n & 3) == 1 || ($n & 3) == 2) {
    #   next;
    # }
    my $turn = $seq->ith($n); # -1); # int(($n-1)/2));
    # print "$turn,"; next;
    my $n2 = Math::BaseCnv::cnv($n,10,2);
    if (is_pow2($n)) { print "\n"; }
    my ($x,$y) = $path->n_to_xy($n);
    my $parity = bird_turn_right($n) ? 1 : 0;
    my $diff = ($parity == $turn ? '' : '  ***');
    printf "%2s %5s %2s,%-2s  %d %s%s\n",
      $n, $n2, $x,$y,  $turn, $parity, $diff;
  }

  # X/(X+Y), (X+Y)/Y high to low both shear only so no change
  # SB Right when floor((N+1)/2 is odd or power 2^k.
  # Right at first and last of row, otherwise LRRL repeat.
  sub Zsb_turn_right {     # bad
    my ($n) = @_;
    $n += ($n&1);
    return (($n & 2) == 1 || ($n & ($n-1)) == 0);
  }
  sub Ysb_turn_right {     # good
    my ($n) = @_;
    if ($n == 3) { return 0; }
    $n = ($n+1) >> 1;
    return (($n & 1) || is_pow2($n));
  }
  # N is 1or2 mod 4, or N=1111or10000 is N or N+1 is pow2
  sub sb_turn_right {     # good
    my ($n) = @_;
    if ($n == 3) {
      return 0;
    }
    if (($n & 3) == 1 || ($n & 3) == 2) {
      return 1;
    }
    return is_pow2($n) || is_pow2($n+1);
  }
  sub XXsb_turn {     # good
    my ($n) = @_;
    ### sb_turn(): "$n  binary ".sprintf('%b',$n)
    my $bit = high_bit($n);
    ### high: "bit=".sprintf('%b',$bit)
    $n -= $bit;
    for ($bit >>= 1; $bit > 2; $bit >>= 1) {
      ### at: "n=".sprintf('%b',$n)." bit=".sprintf('%b',$bit)
      if ($n & $bit) {
        $n -= $bit;
        $n ^= ($bit-1);
      }
      if ($n == $bit-1) {
        return 0;
      }
    }
    return 1;
  }

  sub hcs_turn_right {  # good
    my ($n) = @_;
    return count_1_bits($n+1) & 1;
  }
  sub count_1_bits {
    my ($n) = @_;
    my $count = 0;
    while ($n) {
      $count += ($n & 1);
      $n >>= 1;
    }
    return $count;
  }

  # Y/(X+Y) and (X+Y)/X high to low
  # so transpose on every new bit inserted
  sub bird_turn_right {  # good
    my ($n) = @_;
    if ($n == 2) { return 1; }
    $n++;
    $n >>= 1;
    if ($n == high_bit($n)) {
      return 0;   # first and last of row always 0
    }
    my $ret = bit_length($n) & 1;   # rows alternately 1s and 0s
    if (($n & 3) == 0) {            # but 0mod8 and 7mod8 flip by low 0s
      my $c = count_low_0_bits($n);
      $ret ^= ($c & 1) ^ 1;
    }
    return $ret;
  }
  sub bit_length {
    my ($n) = @_;
    my $len = 0;
    while ($n) {
      $n >>= 1;
      $len++;
    }
    return $len;
  }
  sub count_low_0_bits {
    my ($n) = @_;
    if ($n == 0) { return 0; }
    my $count = 0;
    until ($n % 2) {
      $count++;
      $n /= 2;
    }
    return $count;
  }

  # 0 1 2 3 4 5 6 7 8 9 A B C D E F
  # 1,1,0,1,0,1,0,0,1,1,0,0,1,1,0,0
  #
  sub ayt_turn_right {   # wrong
    my ($n) = @_;
    if (($n & 3) == 1 || ($n & 3) == 2) {
      return 1;
    }
    my $bit = high_bit($n);
    if (bit_length($n) & 1) {
      return ($n == $bit+$bit/2 || $n == $bit+$bit/2-1 ? 1 : 0);
    } else {
      return ($n == $bit || $n == 2*$bit-1 ? 0 : 1);
    }
  }

  sub drib_turn_right {   # wrong
    my ($n) = @_;
    if (($n & 3) == 1 || ($n & 3) == 2) {
      return 1;
    }
    my $bit = high_bit($n);
    if (bit_length($n) & 1) {
      return ($n == $bit+$bit/2 || $n == $bit+$bit/2-1 ? 1 : 0);
    } else {
      return ($n == $bit || $n == 2*$bit-1 ? 0 : 1);
    }
  }

  #            C 3,5 R
  #     A 1,4                                P->A X/X+Y shear North
  #     P 1,3         B 4,3 L                P->B X+Y/Y shear East
  #            Q 3,2         D 5,2           Q->C X/X+Y shear North
  #                                          Q->D X+Y/Y shear East
  #      X=1    X=3    X=4    X=5
  sub cw_turn_right {    # wrong
    my ($n) = @_;
    my $bit = high_bit($n);
    $n -= $bit;
    while ($bit > 2) {
      if ($n & $bit) {
        $n -= $bit;
        $n ^= ($bit-1);
      }
      if ($n == 1 || $n == 2) {
        return 0;
      }
      $bit >>= 1;
    }
    return 1;
    return 0;
  }

  sub to_gray {
    my ($n) = @_;
    my $digits = [ digit_split_lowtohigh($n,2) ];
    Math::PlanePath::GrayCode::_digits_to_gray_reflected($digits,2);
    return digit_join_lowtohigh($digits,2);
  }
  sub from_gray {
    my ($n) = @_;
    my $digits = [ digit_split_lowtohigh($n,2) ];
    Math::PlanePath::GrayCode::_digits_from_gray_reflected($digits,2);
    return digit_join_lowtohigh($digits,2);
  }
  sub sans_high_bit {
    my ($n) = @_;
    return $n ^ high_bit($n);
  }
  sub high_bit {
    my ($n) = @_;
    my $bit = 1;
    while ($bit <= $n) {
      $bit <<= 1;
    }
    return $bit >> 1;
  }

  exit 0;
}

{
  # single row convex hull area
  #
  # convex hull
  # num points = 2n = 8,10,12,14,16
  # area = 8.50,31.50,98.00,280.00,766.00,2049.50,5422.50
  #   dlog=1.762758
  # perimeter =
  #   dlog=0.881831 exp=2.4153
  require Math::NumSeq::Fibonacci;
  require Math::NumSeq::LucasNumbers;
  my $fib = Math::NumSeq::Fibonacci->new;
  my $luc = Math::NumSeq::LucasNumbers->new;

  sub row_hull_area_by_formula {
    my ($depth) = @_;
    return hull_total_by_formula($depth) - $depth**2/2;
    return (F(2*$depth+3) / 2 - $depth**2/2 - $depth - 1);
  }

  require Math::Geometry::Planar;
  require Math::BaseCnv;
  require "$FindBin::Bin/../../path/xt/MyOEIS.pm";
  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
  my @values;
  my $prev_log = 0;
  foreach my $depth (2 .. 9) {
    my $points = [ ];
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
    foreach my $n ($n_row .. $n_end) {
      my ($x,$y) = $path->n_to_xy($n);
      push @$points, [$x,$y];
    }
    if (@$points < 10) {
      print points_str($points),"\n";
    }

    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    my $value;

    {
      if (@$points >= 5) { $planar = $planar->convexhull2; }
      $points = $planar->points;
      my $area = $planar->area;
      my $perimeter = $planar->perimeter;

      $value = $area;
      # $value = $perimeter
      # $value = scalar(@$points);

      my $tf = row_hull_area_by_formula($depth);
      print "$depth area=$area 2*area=",2*$area,"   $tf\n";
    }

    push @values, $value;
    my $num_points = scalar(@$points);
    my $log = $value && log($value);
    my $dlog = $log - $prev_log;
    printf "    points=%d value=$value  log=%.4f dlog=%.6f\n",
      $num_points, $log, $dlog;
    if (@$points <= 10) {
      print "   ",points_str($points),"\n";
    }
    $prev_log = $log;
  }

  shift @values;

  print join(',',map {sprintf '%.2f', $_} @values),"\n";
  print MyOEIS->grep_for_values(array => \@values);
  exit 0;
}


{
  # SB <-> CW points where N gives same X,Y
  # palindromes below high 1-bit

  my $from_path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
  my $to_path = Math::PlanePath::RationalsTree->new(tree_type => 'CW');
  for (my $n = $from_path->n_start; $n < 1000; $n++) {
    my ($x1,$y1) = $from_path->n_to_xy($n);
    my ($x2,$y2) = $to_path->n_to_xy($n);
    if ($x1 == $x2 && $y1 == $y2) {
      printf "%d %b\n", $n, $n;
    }
  }
  exit 0;
}
{
  # row area convex hull
  #
  require Math::NumSeq::Fibonacci;
  require Math::NumSeq::LucasNumbers;
  my $fib = Math::NumSeq::Fibonacci->new;
  my $luc = Math::NumSeq::LucasNumbers->new;


  # x[i]=F[i+2] fibonacci
  # y[i]=x[i+1] + x[i]*(n-i-1)
  #     =F[i+3] + F[i+2]*(n-i-1)
  #
  # x=1                                   1   2
  #  *--------*---* x[i+1],y[i+1]         *---* 5
  #  |        |  /                        |  /
  #  |        | /                         | /
  #  |        |/                          |/
  #  *--------* x[i],y[i]                 * 4
  #
  # A = (x[i+1]-1)*(y[i+1]-y[i]) - (x[i+1]-x[i])*(y[i+1]-y[i])/2
  #   = (x[i+1]-1 - (x[i+1]-x[i])/2) * (y[i+1]-y[i])
  #   = (x[i+1]-1 - x[i+1]/2 + x[i]/2) * (y[i+1]-y[i])
  #   = (x[i+1] + x[i] - 2) * (y[i+1]-y[i]) / 2
  #   = (F[i+3] + F[i+2] - 1) * (y[i+1]-y[i]) / 2
  #   = (F[i+4]-2) * (y[i+1]-y[i]) / 2
  #   = (F[i+4]-2) * (F[i+4] + F[i+3]*(n-(i+1)-1) - (F[i+3] + F[i+2]*(n-i-1)) ) / 2
  #   = (F[i+4]-2) * (F[i+4] + F[i+3]*(n-i-2) - F[i+3] - F[i+2]*(n-i-1) ) / 2
  #   = (F[i+4]-2) * (F[i+4] + F[i+3]*(n-i-2) - F[i+3] - F[i+2]*(n-i-2) - F[i+2]) / 2
  #   = (F[i+4]-2) * (F[i+4] + F[i+3]*(n-i-2) - F[i+4] - F[i+2]*(n-i-2)) / 2
  #   = (F[i+4]-2) * (F[i+3]*(n-i-2) - F[i+2]*(n-i-2)) / 2
  #   = (F[i+4]-2) * F[i+1]*(n-i-2) / 2

  #     2,5 3,5            depth=3
  # 1,4             5,3
  #                 5,2
  #             4,1
  # 1,1

  # fib prod sum
  #
  # P = sum i=0 to n-2  F[i+4]*F[i+1]
  # i=n-2                               i=0 * n-2
  #                                     F[4]*F[1]    n=2
  #                         F[5]*F[2] + F[4]*F[1]    n=3
  #             F[6]*F[3] + F[5]*F[2] + F[4]*F[1]    n=4
  # F[7]*F[4] + F[6]*F[3] + F[5]*F[2] + F[4]*F[1]    n=5
  # F[n+2]*F[n-1]
  #
  # Catalan's identity F[n]^2 - F[n+r]*F[n-r] = F[r]^2 * (-1)^(n-r)
  #
  # F(i+1)^2 = 4*F(i)*F(i-1) + F(i-2)^2              (31)
  # 4*F(i)*F(i-1) = F(i+1)^2 - F(i-2)^2
  # F(i)*F(i-1) = (F(i+1)^2 - F(i-2)^2)/4
  #
  # F(i+4)*F(i+1)
  # = F(i+3)*F(i+1) + F(i+2)*F(i+1)
  # = F(i+1)*F(i+1) + F(i+2)*F(i+1) + F(i+2)*F(i+1)
  # = F(i+1)^2 + F(i+3)^2/2 - F(i)^2/2
  #
  # sum of squares
  # sum i=0 to n of F(i)^2 = F(n)*F(n+1)
  # = sum(i=0,n, F(i)^2)
  #
  # P(n) = sum(i=0,n-2, F(i+4)*F(i+1))
  #      = sum(i=0,n-2, F(i+1)^2 + F(i+3)^2/2 - F(i)^2/2)
  #      = F(n-1)*F(n) + F(n+1)*F(n+2)/2 - F(n-2)*F(n-1)/2 - 1
  #      = (F(n+1)^2 - F(n-2)^2)/4  + (F(n+3)^2 - F(n)^2)/8 - (F(n)^2 - F(n-3)^2)/8 - 1
  # P(n) = F(n+1)^2/4 - F(n-2)^2/4  + F(n+3)^2/8 - F(n)^2/4 + F(n-3)^2/8 - 1

  # PS = sum(i=0,n-2, F(i+4)*F(i+1)*(n-i-2))
  #    = sum(i=0,n-3, F(i+4)*F(i+1)*(n-i-2))
  #                             F(5)*F(2)*1 + F(4)*F(1)*2    n=4
  #               F(6)*F(3)*1 + F(5)*F(2)*2 + F(4)*F(1)*3    n=5
  # F(7)*F(4)*1 + F(6)*F(3)*2 + F(5)*F(2)*3 + F(4)*F(1)*4    n=6
  # PS = sum(i=0,n-1, P(i))
  #    = sum(i=0,n-1, F(i+1)^2/4 - F(i-2)^2/4  + F(i+3)^2/8 - F(i)^2/4 + F(i-3)^2/8 - 1)
  #    = F(n)*F(n+1)/4 - F(n-3)*F(n-2)/4 + F(n+2)*F(n+3)/8 - F(n-1)*F(n)/4 + F(n-4)*F(n-3)/8 - n
  #    = F(n+3)*F(n+2)/8 + F(n+1)*F(n)/4 - F(n)*F(n-1)/4 - F(n-2)*F(n-3)/4 + F(n-3)*F(n-4)/8 - n
  # adjacents

=pod

f2 = f1+f0
f3 = f2+f1
fn1 = f1-f0
fn2 = f0-fn1
fn3 = fn1-fn2
fn4 = fn2-fn3
f3*f2/8 + f1*f0/4 - f0*fn1/4 - fn2*fn3/4 + fn3*fn4/8
= f0*f1
F(n+1)*F(n)

=cut

  #    = F(n+2)^2/8 + F(n+1)^2/8 + F(n+1)*F(n)/8 + F(n+1)*F(n)/4 - F(n)*F(n-1)/4 - F(n-2)*F(n-3)/4 + F(n-3)*F(n-4)/8 - n
  #    = F(n+2)^2/8 + F(n+1)^2/8 + F(n)^2*3/8 + F(n)*F(n-1)*3/8 - F(n)*F(n-1)/4 - F(n-2)*F(n-3)/4 + F(n-3)*F(n-4)/8 - n
  #    = F(n+2)^2/8 + F(n+1)^2/8 + F(n)^2*3/8 + F(n)*F(n-1)/8 - F(n-2)*F(n-3)/4 + F(n-3)*F(n-4)/8 - n
  #    = F(n+2)^2/8 + F(n+1)^2/8 + F(n)^2*3/8 + F(n-1)^2/8 + F(n-1)*F(n-2)/8 - F(n-2)*F(n-3)/4 + F(n-3)*F(n-4)/8 - n
  #    = F(n+2)^2/8 + F(n+1)^2/8 + F(n)^2*3/8 + F(n-1)^2/8 + F(n-2)^2/8 - F(n-2)*F(n-3)/8 + F(n-3)*F(n-4)/8 - n
  #    = F(n+2)^2/8 + F(n+1)^2/8 + F(n)^2*3/8 + F(n-1)^2/8 + F(n-2)^2/8 - F(n-3)^2/8 - n
  #  squares
  #
  #    = F(2*n+2)/8 + F(n+1)^2/8 + F(n)^2*4/8 + F(n-1)^2/8 + F(n-2)^2/8 - F(n-3)^2/8 - n
  #    = F(2*n+2)/8 + F(2*n)/8 + F(n)^2*4/8 + F(n-1)^2*2/8 + F(n-2)^2/8 - F(n-3)^2/8 - n
  #    = F(2*n+2)/8 + F(2*n)/8 + F(n)^2*4/8 + F(2*n-4)*2/8 + F(n-2)^2/8 + F(n-3)^2/8 - n

  # F(n+1)*F(n) = (F(n+1)^2 - F(n-2)^2)/4
  # F(2n) = F(n+1)^2 - F(n-1)^2


  # hull = (F(n+2)-1)^2 - F(n)^2/2 - A(0) - ... - A(n-2)
  # sides = sum(i=0,n-2, (F(i+4)-2) * F(i+1)*(n-i-2) / 2)
  #       = sum(i=0,n-2, (F(i+4)) * F(i+1)*(n-i-2) / 2) - sum(i=0,n-2, F(i+1)*(n-i-2))
  #       = sum(i=0,n-2, F(i+4)*F(i+1)*(n-i-2)/2) - F(n+2) + n + 1
  #       = (F(n)*F(n+1)/4 - F(n-3)*F(n-2)/4 + F(n+2)*F(n+3)/8 - F(n-1)*F(n)/4 + F(n-4)*F(n-3)/8 - n)/2 - F(n+2) + n + 1
  #       = ( + F(n+3)*F(n+2)/8 + F(n)*F(n+1)/4 - F(n-2)*F(n-3)/4 - F(n)*F(n-1)/4 + F(n-3)*F(n-4)/8 - n)/2 - F(n+2) + n + 1
  #       = PS(n)/2  - F(n+2) + n + 1
  #       = (F(n+1)*F(n) - n)/2  - F(n+2) + n + 1
  #       = (F(n+1)*F(n))/2 - F(n+2) + n/2 + 1
  #
  # hull = (F(n+2)-1)^2 - F(n)^2/2 - 2*((F(n+1)*F(n))/2 - F(n+2) + n/2 + 1)
  #      = (F(n+2)-1)^2 - F(n)^2/2 - F(n+1)*F(n) + 2*F(n+2) - n - 2
  #      = F(n+2)^2 + 1 - F(n)^2/2 - F(n+1)*F(n) - n - 2
  #      = F(n+2)^2 - F(n)^2/2 - F(n+1)*F(n) - n - 1

=pod

f2 = f1+f0
f2^2 - f0^2/2 - f1*f0
= f0*f1
= f1^2 + f0*f1 + 1/2*f0^2
= (f0+f1)*f1 + 1/2*f0^2
= f2*f1 + 1/2*f0^2

=cut

  #      = F(n+2)^2 - F(n)^2/2 - F(n+1)*F(n) - n - 1

  sub hull_total_by_formula {
    my ($depth) = @_;

    {
      # F(i)*F(i-1) = (F(i+1)^2 - F(i-2)^2)/4
      # F(i)^2 = (F(2i+2) - F(2i-2) - 2*(-1)^i) / 5

      return (F(2*$depth+3) / 2 - $depth - 1);
    }
    {
      return (F($depth+2) ** 2
              - F($depth+0) ** 2 / 2
              - F($depth+1) * F($depth)
              - $depth
              - 1);
    }
    {
      my $total = (F($depth+2) - 1) ** 2;
      $total -= F($depth+0) ** 2 / 2;

      my $sides = F($depth+1)*F($depth)/2
        - F($depth+2) + $depth/2 + 1;
      return $total - 2*$sides;
    }
    {
      my $total = (F($depth+2) - 1) ** 2;
      $total -= F($depth+0) ** 2 / 2;

      $total -= F($depth)*F($depth+1) / 4;
      $total -= - F($depth-2)*F($depth-3) / 4;
      $total -= F($depth+3)*F($depth+2) / 4;
      $total -= - F($depth)*F($depth-1) / 4;
      $total -= F($depth-3)*F($depth-4) / 8;
      $total -= 2*(- F($depth+2) + $depth + 1);
      return $total;
    }
    {
      my $total = (F($depth+2) - 1) ** 2;
      $total -= F($depth+0) ** 2 / 2;

      foreach my $i (0 .. $depth-2) {
        ### at: "depth=$depth i=$i"
        ### sub: hull_side_area_by_formula($depth,$i)
        $total -= 2*hull_side_area_by_formula($depth,$i);
      }
      return $total;
    }
  }
  sub hull_total_by_sum {
    my ($depth) = @_;
    my $total = (F($depth+2) - 1) ** 2;
    $total -= F($depth+0) ** 2 / 2;
    foreach my $i (0 .. $depth-2) {
      ### at: "depth=$depth i=$i"
      ### sub: hull_side_area_by_formula($depth,$i)
      $total -= 2*hull_side_area_by_formula($depth,$i);
    }
    return $total;
  }
  sub hull_side_area_by_formula {  # from i to i+1
    my ($depth, $i) = @_;
    $i <= $depth-2 or die;

    return (F($i+4) - 2) * F($i+1) * ($depth-$i-2) / 2;

    return (F($i+4) - 2) * (F($i+4) + F($i+3)*($depth-$i-2)
                                    - F($i+3) - F($i+2)*($depth-$i-1)) / 2;

    {
      my ($x1,$y1) = hull_xy_by_formula($depth,$i);
      my ($x2,$y2) = hull_xy_by_formula($depth,$i+1);
      return ($x2+$x1-2)*($y2-$y1)/2;
    }
    {
      my ($x1,$y1) = hull_xy_by_formula($depth,$i);
      my ($x2,$y2) = hull_xy_by_formula($depth,$i+1);
      return ($x2-1)*($y2-$y1) - ($x2-$x1)*($y2-$y1)/2;
    }
  }

  #
  # F[n-1],F[n]
  #  *---*               M = F[n-1]*F[n]/2
  #   \  |
  #    \ |
  #      * F[n],F[n-1]
  sub hull_xy_by_formula {
    my ($depth, $i) = @_;
    my $x = F($i+2);
    return ($x,
            $x*($depth-$i-1) + F($i+3));
  }
  sub hull_side_area_by_xy {  # from i to i+1
    my ($depth, $i) = @_;
    $i <= $depth-2 or die;
    my ($x1,$y1) = hull_xy_by_formula($depth,$i);
    my ($x2,$y2) = hull_xy_by_formula($depth,$i+1);
    my $h = $y2 - $y1;
    $x2 >= $x1 or die;
    $y2 >= $y1 or die "depth=$depth i=$i  y1=$y1  y2=$y2";
    $h >= 0 or die;
    return ($x1-1) * $h + ($x2-$x1)*$h/2;
  }

  require Math::Geometry::Planar;
  require Math::BaseCnv;
  require "$FindBin::Bin/../../path/xt/MyOEIS.pm";
  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
  my @values;
  my $prev_log = 0;
  foreach my $depth (1 .. 9) {
    print "depth=$depth\n";
    my $points = [
                  # with 0,0 1,0 0,1 is F[2n-3]/2
                  # [1,0], [0,1],[0,0],

                  [1,1],
                  [0,0],
                  # [1,1], [0,1], [1,0]
                 ];
    my $n_row = $path->tree_depth_to_n($depth);
    foreach my $n ($path->tree_depth_to_n($depth) ..
                   $path->tree_depth_to_n_end($depth)) {
      my ($x,$y) = $path->n_to_xy($n);
      push @$points, [$x,$y];
    }
    if (@$points < 10) {
      print points_str($points),"\n";
    }

    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    my $value;

    # {
    #   # bbox, max(X) = Pell 12,29,70
    #   #       max(Y) = Pell 5,12,29
    #   #       area = pell*pell = 297,1904,11523,68208,400081
    #   $planar = $planar->bbox;
    #   $points = $planar->points;
    #   # $value = $points->[0]->[0];
    #    $value = $planar->area;
    # }

    {
      # convex hull
      # num points = 2n = 8,10,12,14,16
      # area = 0,2,139,834,4892,28551,166453
      #   dlog=1.762758
      # perimeter = 60.88, 152.99, 375.30, 912.00, 2207.67, 5335.71, 12887.44
      #   dlog=0.881831 exp=2.4153
      if (@$points >= 5) { $planar = $planar->convexhull2; }
      $points = $planar->points;
      my $area = $planar->area;
      $value = $area;
      # $value = $planar->perimeter;
      # $value = scalar(@$points);

      # foreach my $p (reverse @$points) {
      #   my $n = $path->xy_to_n(@$p);
      #   # if ($n >= $n_row) { $n -= $n_row; }
      #   printf "  xy=%4d,%4d  n=%8s [%8b]\n",
      #     $p->[0],$p->[1], $n,$n;
      # }

      foreach my $i (0 .. $depth) {
        my $p = $points->[-1-$i];
        my ($x,$y) = hull_xy_by_formula($depth,$i);

        my $a = $i <= $depth-2 ? hull_side_area_by_xy($depth,$i) : -9999;
        my $af = $i <= $depth-2 ? hull_side_area_by_formula($depth,$i) : -9999;
        printf "  xy=%4d,%4d  formula=%4d,%4d  %s %s\n",
          $p->[0],$p->[1], $x,$y, $a, $af;
      }

      my $tsum = hull_total_by_sum($depth);
      my $tf = hull_total_by_formula($depth);
      print "$depth area=$area 2*area=",2*$area,"   $tsum $tf\n";
      my $f2d = F(2*$depth+3)/2;
      print "  F(2d+3)/2 = $f2d\n";
    }

    # {
    #   # perimeter = 140.57, 774.39, 4451.85, 25868.97, 150680.50, 878118.84
    #   #   dlog=1.762722
    #    $value = $planar->perimeter * 139 / 140.57;
    #   # $value = $planar->centroid()->[0];
    # }

    push @values, $value;
    my $num_points = scalar(@$points);
    my $log = $value && log($value);
    my $dlog = $log - $prev_log;
    printf "    points=%d value=$value  log=%.4f dlog=%.6f\n",
      $num_points, $log, $dlog;
    if (@$points <= 10) {
      print "   ",points_str($points),"\n";
    }
    $prev_log = $log;
  }

  shift @values;

  print join(',',map {sprintf '%.2f', $_} @values),"\n";
  print MyOEIS->grep_for_values(array => \@values);
  exit 0;

}

sub planar_bbox_xy {
  my ($planar) = @_;
  my $points = $planar->bbox->points;
  return (@{$points->[3]},
          @{$points->[1]});
}    

# Return a Math::Geometry::Planar which is the convex hull around the SB
# tree points to $depth.  $type is "whole" or "row".
sub hull_planar {
  my ($depth, $type) = @_;
  $type ||= 'whole';
  ### hull_planar(): $type
  my $points = [];
  if ($type eq 'row') {
  } elsif ($type eq 'whole') {
    push @$points, [1,1];
  } else {
    die "hull_planar() unrecognised type: ",$type;
  }
  my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
  my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
  foreach my $n ($n_row ... $n_end) {
    my ($x,$y) = $path->n_to_xy($n);
    push @$points, [$x,$y];
  }
  ### num points: scalar(@$points)
  if (@$points >= 1 && @$points < 3) {
    push @$points, $points->[-1];
    push @$points, $points->[-1];
  }
  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  $planar->points($points);
  if (@$points >= 5) {
    $planar = $planar->convexhull2;
  }
  return $planar;
}
sub hull_points_aref {
  my ($depth, $type) = @_;
  my $planar = hull_planar($depth,$type);
  return $planar->points;
}

{
  my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
  my @values;
  foreach my $depth (5) {
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
    my $width = $n_end - $n_row + 1;
    $n_end = $n_row + $width/2 - 1;
    foreach my $n ($n_row + 1 .. $n_end - 1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      if ($dx >= 2 && $dy >= 0 && $x < $y) {
        print "\n$x,$y $x2,$y2\n";
        exit;
      }
      print "$dy,";
      push @values, $dy;
    }
    print "\n";
  }
  print join(',',@values),"\n";
  require "$FindBin::Bin/../../path/xt/MyOEIS.pm";
  print MyOEIS->grep_for_values(array => \@values);
  exit 0;
}

{
  # coprime points colinear in between
  foreach my $p1 (1 .. 20) {
    foreach my $q1 (1 .. 20) {
      foreach my $p2 ($p1 .. 20) {
        foreach my $q2 (1 .. 20) {

          foreach my $p ($p1+1 .. $p2-1) {
            # (p-p1)/(q-q1) = (p2-p1)/(q2-q1)
            # (q-q1) = (p-p1)*(q2-q1)/(p2-p1)
            # q = (p-p1)*(q2-q1)/(p2-p1) + q1
            # q = ((p-p1)*(q2-q1) + (p2-p1)*q1)/(p2-p1)
            # q = ( (p-p1)*q2-(p-p1)*q1 + (p2-p1)*q1 )/(p2-p1)
            # q = ( p*q2-p1*q2 - p*q1 + p1*q1 + p2*q1 - p1*q1 )/(p2-p1)
            # q = ( p*q2 - p1*q2 - p*q1 + p2*q1 )/(p2-p1)

            my $qnum = ($p-$p1)*($q2-$q1) + ($p2-$p1)*$q1;
            my $qden = $p2-$p1;
            if ($qnum % $qden == 0) {
              print "$p1,$q1 to $p2,$q2   p=$p  q=$qnum/$qden\n";
              exit;
            }
          }
        }
      }
    }
  }
  exit 0;
}

{
  # Pick area of lattice polygon
  # area = I + B/2 - 1
  # I = num interior points
  # B = num boundary points
  # inside(d) = area(d) - boundary(d)/2 + 1

  # to level d inclusive
  #
  #  1,1 *         boundary=1      d=0

  #  1,2 *         boundary=3      d=1
  #      |\        area = 1/2
  #  1,1 *-*       inside = 0
  #        2,1       1/2 == 0 + 3/2 - 1

  #   1,3  2,3      boundary=7
  #     *-*         area = 3+1/2
  #     |  \        inside = 1
  # 1,2 *   * 3,2     7/2 == 1 + 7/2 - 1
  #     |   |
  #     *-*-*
  #   1,1   3,1
  #      2,1

  # boundary(d) = if(d==0, 1, 2^d + 2*d - 1)
  #             = 1, 3, 7, 13, 23, 41, 75, 141, 271,
  # area(d) = 5/2*(2^d-1)-2*d
  #
  # 5/2*(2^d-1)-2*d = inside + (2^d + 2*d + 1)/2 - 1
  # inside(d) = 5/2*(2^d-1)-2*d - (2^d + 2*d - 1)/2 + 1
  #           = (5*2^d - 5 - 2^d - 2*d + 1)/2 - 2*d  + 1
  #           = (4*2^d - 4 - 2*d)/2 - 2*d  + 1
  #           = 2*2^d - 2 - d - 2*d  + 1
  # inside(d) = 2*2^d - 3*d - 1
  # inside(d)+boundary(d) = 2*2^d - 3*d - 1  +  2^d + 2*d - 1
  #                       = 3*2^d - d - 2
  # A079583 3*2^n-n-2
  # for(d=1,d=8, print1(inside(d),", "))

  # total points T = sum 2^d = 2^(d+1)-1
  # T = I+B
  # I = 2^(d+1)-1 - B
  # 5/2*(2^d-1)-2*d = 2^(d+1)-1 - B/2 - 1
  # B/2 = 2^(d+1)-1 - 1 - 5/2*(2^d-1) + 2*d
  #     = 2^(d+1)-1 - 1 - 5/2*2^d + 5/2 + 2*d
  #     = 2*2^d - 1 - 1 - 5/2*2^d + 5/2 + 2*d
  #     = -1/2*2^d + 2*d + 1/2

  # between rows

  #   1,3  2,3        d=1 to d=2
  #     *-*           boundary=6
  #     | |\          area = 3           3 == 3*2^0-
  # 1,2 *---* 3,2     inside = 1
  #      \| |         3 == 1 + 6/2 - 1
  #       *-*
  #   1,1   3,1
  #      2,1
  #
  # area(d) = 5*2^(d-1) - 2         /* d to d+1 */
  # boundary(d) = 2^d + 2^(d+1)     /* d and d+1 */
  # boundary(d) = 3*2^d
  # inside(d) = area(d) - boundary(d)/2 + 1
  #           = 5*2^(d-1) - 2 - 3*2^d/2 + 1
  #           = 5*2^(d-1) - 2 - 3*2^(d-1) + 1
  #           = 2*2^(d-1) - 1
  #           = 2^d - 1
  # inside replicates
  #   inside(d+1) = 2*inside(d) + 1
  # middle point d,d replicate
  #       d,2d          2d,d
  #    d,3d  3d,2d  2d,3d  3d,d

  require Math::PlanePath::RationalsTree;
  require Math::Geometry::Planar;
  require "$FindBin::Bin/../../path/xt/MyOEIS.pm";
  my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
  my @values;
  foreach my $depth (1 .. 8) {
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
    my ($n_prev_row, $n_prev_end) = $path->tree_depth_to_n_range($depth+1);
    my $points
      = [ (map {[$path->n_to_xy($_)]} $n_row .. $n_end),
          # [1,1],
           reverse(map {[$path->n_to_xy($_)]} $n_prev_row .. $n_prev_end)
        ];

    if (@$points <= 10) {
      print "   ",points_str($points),"\n";
    }

    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    my $area = $planar->area;

    my $x_max = max(map {$_->[0]} @$points);
    my $y_max = max(map {$_->[1]} @$points);

    my $num_points = scalar(@$points);
    my $boundary = 0;
    my $inside_and_boundary = 0;
    foreach my $x (0 .. $x_max) {
      foreach my $y (0 .. $y_max) {
        my $p = [$x,$y];
        if ($planar->isinside($p)) {
          $inside_and_boundary++;
        }
        if (planar_isboundary($planar, $p)) {
          $boundary++;
          if (@$points <= 3) {
            print "boundary $x,$y\n";
          }
        }
      }
    }
    my $inside = $inside_and_boundary - $boundary;
    my $inside_by_formula = $area - $boundary/2 + 1;

    print "$depth  area=$area  points=$num_points inside+boundary=$inside_and_boundary boundary=$boundary inside=$inside=[f]$inside_by_formula\n";
    # print "   $total $total_formula $total_recurrence\n";
    # printf "  %6b  %6b\n", $area, $total;
    # push @values, $inside_and_boundary;
    push @values, $boundary;
  }
  shift @values;
  print join(',',@values),"\n";
  print MyOEIS->grep_for_values(array => \@values);
  exit 0;

  # return the number of integer points on the boundary of $planar
  sub planar_count_boundary_integers {
    my ($planar) = @_;
    my ($x_min, $y_min, $x_max, $y_max) = planar_bbox_xy($planar);
    ### planar_count_boundary_integers() ...
    ### extents: "$x_min to $x_max  and  $y_min to $y_max"
    my $count = 0;
    foreach my $x ($x_min .. $x_max) {
      foreach my $y ($y_min .. $y_max) {
        my $p = [$x,$y];
        if (planar_isboundary($planar, $p)) {
          ### found: join(', ',@$p)
          $count++;
        }
      }
    }
    ### $count
    return $count;
  }
}



{
  # sum squares
  require Math::NumSeq::Fibonacci;
  my $seq = Math::NumSeq::Fibonacci->new;
  my $sum = 0;
  foreach (1 .. 20) {
    my ($i, $value) = $seq->next;
    $sum += $value**2;
    my $si = $seq->_UNTESTED__value_to_i($sum*2+1) // 'none';
    print "$sum  $si\n";
  }
  exit 0;
}

{
  # Czyz and Self, num-1 at powers of 2
  # HCS
  #   A229742   X, extra initial 0/1
  #   A071766   Y
  #   A071585   X+Y sum

  # A071585 = X+Y starts offset=0 X+Y=1
  # A071766 = Y   starts offset=0 Y=1,1,1,2,1,2,3,3,1,2,3,3,4
  # A209229 Characteristic function of powers of 2, cf. A000079.
  # A036987 characteristic a(n)=1 iff n = 2^m - 1 offset=0 start 1,1,0,1,

  # num = A229742(n)-A209229(n) for n>=1
  # den = A071766(n) for n>=1

  # n=1  2   3   4   5    6    7   8   9   10   11   12   13   14   15  16
  #   0, 1, 1/2, 2, 3/2, 1/3, 2/3, 3, 5/2, 4/3, 5/3, 1/4, 2/5, 3/4, 3/5, 4,

  require Math::NumSeq::OEIS::File;
  my $hcs_num = Math::NumSeq::OEIS::File->new(anum=>'A229742');
  my $pow = Math::NumSeq::OEIS::File->new(anum=>'A209229');
  # my $pow = Math::NumSeq::OEIS::File->new(anum=>'A036987');
  my $den = Math::NumSeq::OEIS::File->new(anum=>'A071766');

  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'HCS');
  foreach my $n (1 .. 16) {
    my ($x,$y) = $path->n_to_xy($n);
    if ($y == 1) {
      $x--;
    }

    if ($y == 1) {
      print "$x";
    } else {
      print "$x/$y";
    }
    print ", ";

    my $xs = $hcs_num->ith($n) - $pow->ith($n);
    # if (is_pow2($n)) { $xs--; }
    my $ys = $den->ith($n);

    $xs == $x && $ys == $y or die "n=$n  $x/$y seqs $xs/$ys  ";
  }
  print "\n";

  print "nums ";
  foreach my $n (1 .. 32) {
    my ($x,$y) = $path->n_to_xy($n);
    if ($y == 1) {
      $x--;
    }
    print "$x,";

    my $xs = $hcs_num->ith($n) - $pow->ith($n);
    # if (is_pow2($n)) { $xs--; }
    my $ys = $den->ith($n);

    $xs == $x && $ys == $y or die "n=$n  $x/$y seqs $xs/$ys  ";
  }
  print "\n";
  exit 0;
}



{
  # Planar contour isinside()
  require Math::Geometry::Planar;
  my $contour = Math::Geometry::Planar->new;
  $contour->polygons([
                      [[0,0], [4,0], [4,4], [0,4]],   # outer
                      [[0,0], [0,2], [2,2], [2,0]],   # hole
                     ]);
  print "area ",$contour->area,"\n";
  print "perimeter ",$contour->perimeter,"\n";
  my $step = .25;
  $step = 1;
  for (my $y = 0; $y <= 4; $y += $step) {
    for (my $x = 0; $x <= 4; $x += $step) {
      print $contour->isinside([$x,$y]) ? '*' : '_';
    }
    print "\n";
  }    
  exit 0;
}



{
  # row area of RationalsTree
  # between: 0.5, 3,8,18,38,78,158,318,638

  # L = (1 1)   R = (1 0)
  #     (0 1)       (1 1)
  #
  # r[n+1] = 2*r[n] + gap
  #
  # L^n * (1) = (n+1)   R^n * (1) = ( 1 )
  #       (1)   ( 1 )         (1)   (n+1)
  #
  # L.R^n.[1;1] = [n+2; n+1]   L.R^(n+1).[1;1] = [n+3; n+2]
  # R.L^n.[1;1] = [n+1; n+2]   R.L^(n+1).[1;1] = [n+2; n+3]
  #
  #         +2,+3 *             area = 2*2/2 = 2
  #              / \
  #             /   \
  #     +1,+2  *     * +3,+2
  #             \   /
  #              \ /
  #               *
  #             +2,+1
  #
  # r[n+1] = 2*r[n] + 2
  #   3 == 2*.5 + 2
  #   8 == 2*3 + 2
  #   18 == 2*8 + 2
  #   38 == 2*18 + 2

  # r[n] = 2 + 2*(2 + 2*(2 + 2*(... 0.5)))
  #      = 2*(1 + .. + 2^(n-1)) + 2^(n-1)*.5
  #      = 2*(2^n - 1) + 2^(n-1)
  #      = 4*2^(n-1) - 2 + 2^(n-1)
  #      = 5*2^(n-1) - 2
  # for(n=0,8,print(5*2^(n-1) - 2,","))
  #
  # Cumulative
  # R[n] = sum r[0] to r[n-1]
  #      = 5*(2^-1 + 1 + ... + 2^(n-2)) - 2n
  #      = 5/2 + 5*(2^(n-1) - 1) - 2n
  #      = 5*(1/2 + 2^(n-1) - 1) - 2n
  #      = 5*(2^(n-1) - 1/2) - 2n
  #      = 5/2*(2^n - 1) - 2n
  #
  require Math::PlanePath::RationalsTree;
  require Math::Geometry::Planar;
  require "$FindBin::Bin/../../path/xt/MyOEIS.pm";
  my $path = Math::PlanePath::RationalsTree->new (tree_type => 'SB');
  my @values;
  my @prev_points = ([1,1]);
  my $total = 0;
  foreach my $depth (1 .. 9) {
    my ($n_row, $n_end) = $path->tree_depth_to_n_range($depth);
    my @points = map {[$path->n_to_xy($_)]} $n_row .. $n_end;

    my $planar = Math::Geometry::Planar->new;
    my $points = [ @points, reverse @prev_points ];
    $planar->points($points);
    my $area = $planar->area;
    $total += $area;

    my $area_formula = rationals_area_between_by_formula($depth-1);
    my $area_recurrence = rationals_area_between_by_recurrence($depth-1);
    my $total_formula = rationals_area_total_by_formula($depth);
    my $total_recurrence = rationals_area_total_by_recurrence($depth);

    print "$depth  $area $area_formula $area_recurrence\n";
    print "   $total $total_formula $total_recurrence\n";
    printf "  %6b  %6b\n", $area, $total;
    if (@points <= 10) {
      print "   ",points_str($points),"\n";
    }
    @prev_points = @points;

    push @values, $total+.5;
  }

  shift @values;
  print join(',',@values),"\n";
  print MyOEIS->grep_for_values(array => \@values);
  exit 0;

  sub rationals_area_between_by_formula {
    my ($depth) = @_;
    return 5*2**($depth-1) - 2;
  }
  sub rationals_area_between_by_recurrence {
    my ($depth) = @_;
    if ($depth < 0) { die; }
    if ($depth == 0) { return 1/2; }
    return 2*rationals_area_between_by_recurrence($depth-1) + 2;
  }
  BEGIN { memoize('rationals_area_between_by_recurrence') }

  sub rationals_area_total_by_formula {
    my ($depth) = @_;
    return 5/2 * (2**$depth - 1) - 2*$depth;
  }
  sub rationals_area_total_by_recurrence {
    my ($depth) = @_;
    if ($depth < 0) { die; }
    if ($depth == 0) { return 0; }
    return 2*rationals_area_total_by_recurrence($depth-1) + 2*$depth - 3/2;
  }
  BEGIN { memoize('rationals_area_total_by_recurrence') }
}
{
  # Kepler harmonics in other trees

  my %xy_is_kepler = ('1/6' => 1, # 7 of
                      '1/5' => 2,
                      '5/6' => 1,
                      '1/4' => 2,
                      '4/5' => 1,
                      '1/3' => 2,
                      '3/4' => 1,

                      '1/1' => 1,

                      '1/2' => 2, # 6 of
                      '2/5' => 1,
                      '2/3' => 2,
                      '3/8' => 1,
                      '3/5' => 2,
                      '5/8' => 1,
                     );

  my $filter = sub {
    my ($x,$y) = @_;
    my $str = "$x/$y";
    return (($xy_is_kepler{$str}||0) == 1 ? $str : '');
  };
  sub print_as_fractions {
    my ($path) = @_;

    my $n = $path->n_start;
    foreach (1) {
      my ($x,$y) = $path->n_to_xy($n++);
      print centre($filter->($x,$y),64);
    }
    print "\n";

    print "                 /------------- -------------\\\n";
    foreach (1 .. 2) {
      my ($x,$y) = $path->n_to_xy($n++);
      print centre($filter->($x,$y),32);
    }
    print "\n";

    print "         /----   ----\\                   /----   ----\\\n";
    foreach (1 .. 4) {
      my ($x,$y) = $path->n_to_xy($n++);
      print centre($filter->($x,$y),16);
    }
    print "\n";

    print "     /   \\           /   \\           /   \\           /   \\\n";
    foreach (1 .. 8) {
      my ($x,$y) = $path->n_to_xy($n++);
      print centre($filter->($x,$y),8);
    }
    print "\n";

    print " /   \\   /   \\   /   \\   /   \\   /   \\   /   \\   /   \\   /   \\\n";
    foreach (16 .. 31) {
      my ($x,$y) = $path->n_to_xy($n++);
      print centre($filter->($x,$y),4);
    }
    print "\n";

    print "\n";
  }

    sub centre {
      my ($str, $width) = @_;
      my $extra = max (0, $width - length($str));
      my $left = int($extra/2);
      my $right = $extra - $left;
      return ' 'x$left . $str . ' 'x$right;
    }

    sub xy_to_cfrac_str {
      my ($x,$y) = @_;
      my @quotients;
      while ($x > 0 && $y > 0) {
        push @quotients, int($x/$y);
        $x %= $y;
        ($x,$y) = ($y,$x);
      }
      return "[".join(',',@quotients)."]";
    }

    #------------------------------------------------------------------------------

    my $rationals_type_arrayref
      = Math::PlanePath::RationalsTree->parameter_info_hash()->{'tree_type'}->{'choices'};
    my $fractions_type_arrayref
      = Math::PlanePath::FractionsTree->parameter_info_hash()->{'tree_type'}->{'choices'};

    print "RationalsTree\n";
    print "-------------\n\n";

    foreach my $tree_type (@$rationals_type_arrayref) {
      print "$tree_type tree\n";

      my $path = Math::PlanePath::RationalsTree->new
        (tree_type => $tree_type);
      print_as_fractions ($path);
    }

    print "\n";
    print "FractionsTree\n";
    print "-------------\n\n";

    foreach my $tree_type (@$fractions_type_arrayref) {
      print "$tree_type tree\n";

      my $path = Math::PlanePath::FractionsTree->new
        (tree_type => $tree_type);
      print_as_fractions ($path);
    }

  }

  {
    # Benson homomorphisms
    require Math::PlanePath::FractionsTree;
    my $path = Math::PlanePath::FractionsTree->new;
    { my ($x,$y) = $path->n_to_xy(0b1);
      print "$x $y\n";
    }
    { my ($x,$y) = $path->n_to_xy(0b1_1);
      print "$x $y\n";
    }
    { my ($x,$y) = $path->n_to_xy(0b1_10);
      print "$x $y\n";
    }
    { my ($x,$y) = $path->n_to_xy(0b1_101);
      print "$x $y\n";
    }
    { my ($x,$y) = $path->n_to_xy(0b1_1011);
      print "$x $y\n";
    }

    { my ($x,$y) = $path->n_to_xy(0b1);
      print "$x $y\n";
    }
    { my ($x,$y) = $path->n_to_xy(0b1_1);
      print "$x $y\n";
    }
    { my ($x,$y) = $path->n_to_xy(0b1_11);
      print "$x $y\n";
    }
    { my ($x,$y) = $path->n_to_xy(0b1_110);
      print "$x $y\n";
    }
    { my ($x,$y) = $path->n_to_xy(0b1_1101);
      print "$x $y\n";
    }
    exit 0;
  }



{
  # convex hull slopes
  #
  require Math::Geometry::Planar;
  require Math::BigRat;
  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
  my @values;
  my $prev_log = 0;
  foreach my $depth (1 .. 15) {
    print "depth=$depth\n";
    my $points = [ [1,1] ];
    my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      push @$points, [$x,$y];
    }

    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    if (@$points > 3) { $planar = $planar->convexhull2; }
    $points = $planar->points;

    foreach my $i (0 .. $#$points) {
      my $p = $points->[$i];
      my $r = $points->[$i-1];
      my $dx = abs($p->[0] - $r->[0]);
      my $dy = abs($p->[1] - $r->[1]);
      my $slope = Math::BigRat->new("$dy/$dx");
      print " $slope";
    }
    print "\n";
  }
  exit 0;
}

{
  # parallelogram tiling tikz picture
  #
  my $x_max = 16;
  my $y_max = 16;

  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');

  my $draw_line = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    ($x1,$y1, $x2,$y2) = line_clip($x1,$y1, $x2,$y2) or return;
    foreach my $v ($x1,$y1, $x2,$y2) {
      if ($v != int($v)) {
        $v = sprintf '%.3f', $v;  # mutate variables
      }
    }
    print "  \\draw ($x1, $y1) -- ($x2, $y2);\n";
  };

  my $draw_poly = sub {
    my @coords = @_;
    convex_poly_clip(\@coords);
    if (@coords) {
      print "  \\filldraw ";
      while (@coords) {
        my $x = shift @coords;
        my $y = shift @coords;
        print "($x,$y)--";
      }
      print "cycle;\n";
    }
  };

  my $max_depth = 0;
  foreach my $x (0 .. $x_max) {
    foreach my $y (0 .. $y_max) {
      my $n = $path->xy_to_n($x,$y) // next;
      {
        my ($n_L, $n_R) = $path->tree_n_children($n);
        my ($x_L,$y_L) = $path->n_to_xy($n_L) or die;
        my ($x_R,$y_R) = $path->n_to_xy($n_R) or die;

        $draw_poly->($x,$y,
                     $x_L,$y_L,
                     $x_R,$y_R);
      }
      my $depth = $path->tree_n_to_depth($n);
      my $n_start = $path->tree_depth_to_n($depth);
      my $offset = $n - $n_start;
      $max_depth = max($max_depth, $depth);
    }
  }

  # print "  % max_depth $max_depth\n";
  foreach my $depth (0 .. $max_depth) {
    my ($prev_x,$prev_y);
    foreach my $n ($path->tree_depth_to_n($depth) .. $path->tree_depth_to_n_end($depth)) {
      my ($x,$y) = $path->n_to_xy($n) or die;
      if (defined $prev_x) {
        $draw_line->($prev_x,$prev_y, $x,$y);
      }
      $prev_x = $x;
      $prev_y = $y;
    }
  }

  # return new ($x1,$y1, $x2,$y2) which is clipped to $x_max,$y_max
  # return empty list if all outside
  sub line_clip {
    my ($x1,$y1, $x2,$y2) = @_;
    ### line_clip(): "$x1,$y1, $x2,$y2  within $x_max,$y_max"
    return if $x1 > $x_max && $x2 > $x_max;
    return if $y1 > $y_max && $y2 > $y_max;

    my ($x1new,$y1new) = line_end_clip($x2,$y2, $x1,$y1, $x_max,$y_max);
    ($x2,$y2) = line_end_clip($x1,$y1, $x2,$y2, $x_max,$y_max);
    return ($x1new,$y1new, $x2,$y2);
  }

  # return new $x2,$y2 which is clipped to $x_max,$y_max
  sub line_end_clip {
    my ($x1,$y1, $x2,$y2, $x_max,$y_max) = @_;
    ### _line_end_clip(): "$x1,$y1, $x2,$y2 within $x_max,$y_max"

    my ($x2new, $y2new);
    if ($x2 > $x_max) {
      # x2 is outside LO to HI, clip to x2=HI and y2 set to corresponding
      my $xlen = $x2 - $x1;

      $x2new = $x_max;
      $y2new = ($y2*($x2new-$x1) + $y1*($x2-$x2new)) / $xlen;
      ### x clip: "to $x2new,$y2new"
    } else {
      $x2new = $x2;
      $y2new = $y2;
    }

    if ($y2new > $y_max) {
      my $ylen = $y2 - $y1;
      $y2new = $y_max;
      $x2new = ($x2*($y2new-$y1) + $x1*($y2-$y2new)) / $ylen;
      ### y clip: "to $x2new,$y2new   left ".($y2new-$y1)." right ".($y2-$y2new)
    }

    return ($x2new,$y2new);
  }

  sub convex_poly_clip {
    my ($aref) = @_;
    ### convex_poly_clip(): $aref

    foreach my $side (1 .. 4) {  # each side
      ### side: $_
      my $max = ($side & 1 ? $y_max : $x_max);

      for (my $i = 0; $i < $#$aref && $#$aref >= 3; ) {
        ### at: "i=$i of ".scalar(@$aref)."  ".join(', ',@$aref)
        my $y = $aref->[$i+1];
        if ($y <= $max) {
          # This vertex is below the $max limit, keep it unchanged.
          $i += 2;

        } else {
          # This vertex is outside the $max limit, replace it by zero, one or
          # two new clipped points.
          my ($x,$y) = splice @$aref, $i,2;

          {
            my $yprev = $aref->[$i-1];  # with possible wrap back to $xn,$yn
            if ($yprev <= $max) {
              my $xprev = $aref->[$i-2];
              my $xnew = ($x*($max - $yprev) + $xprev*($y - $max)) / ($y-$yprev);
              splice @$aref, $i,0, $xnew,$max;
              $i += 2;
            } else {
              # $yprev and $y both above $max limit, so nothing for segment
              # $yprev to $y, just leave $yprev for the next vertex to
              # consider.  (This case only occurs when $i==0 and so $yprev is
              # wrapped back to the last vertex $yn.  Any later $i will have
              # $yprev already clipped to $yprev<=$max.)
            }
          }

          {
            my $inext = $i % scalar(@$aref);
            my $ynext = $aref->[$inext+1];
            if ($ynext <= $max) {
              my $xnext = $aref->[$inext];
              my $xnew = ($x*($max - $ynext) + $xnext*($y - $max)) / ($y-$ynext);
              splice @$aref, $i,0, $xnew,$max;
              $i += 2;
            } else {
              # $y and $ynext both above $max limit, so nothing for segment $y
              # to $ynext
            }
          }
        }
      }

      # rotate 90
      for (my $i = 0; $i < $#$aref; $i += 2) {
        ($aref->[$i],$aref->[$i+1]) = ($aref->[$i+1], -1 - $aref->[$i]);
      }
    }
    if (@$aref == 2) {
      @$aref = ();
    }
  }

  exit 0;
}



{
  # Bird tree turn sequence prints

  require Math::NumSeq::PlanePathTurn;
  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'Bird');
  my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                             turn_type => 'Left');
  foreach my $depth (2 .. 7) {
    my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);
    foreach my $n ($n_lo .. $n_hi) {
      print $seq->ith($n);
    }
    print "\n";

    foreach (1, 2) {
      my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth - 1);
      foreach my $n ($n_lo .. $n_hi) {
        print 1 - $seq->ith($n);
      }
    }
    print "\n";
    print "\n";
  }
  exit 0;
}

{
  # Bird tree turn sequence as opposites

  require Math::NumSeq::PlanePathTurn;
  my $path = Math::PlanePath::RationalsTree->new(tree_type => 'Bird');
  my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                             turn_type => 'Right');
  my $count = 0;
  foreach my $depth (2 .. 20) {
    my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);
    my ($prev_n_lo, $prev_n_hi) = $path->tree_depth_to_n_range($depth-1);
    my $prev_n = $prev_n_lo;
    foreach my $n ($n_lo .. $n_hi) {
      my $turn = $seq->ith($n);
      my $prev_turn = $seq->ith($prev_n);
      if ($n != $n_lo && $n != $n_hi) {
        if ($turn != 1 - $prev_turn) {
          print "depth=$depth $n not opp $turn $prev_turn\n";
        }
      }

      $prev_n++;
      if ($prev_n > $prev_n_hi) {
        $prev_n = $prev_n_lo;
      }
      $count++;
    }
  }
  print "$count\n";
  exit 0;
}

{
  # Johnston

  # 1,0 0,1                          mediant between each
  # 1,0 1,1 0,1
  # 1,0 2,1 1,1 1,2 0,1
  # 1,0 3,1 2,1 3,2 1,1 2,3 1,2 1,3 0,1
  #
  # 1,1         unrepeated
  # 2,1 1,2
  # 3,1 3,2 2,3 1,3
  # 
  exit 0;
}

{
  # fib sum 
  #
  # S = sum i=0 to n-2  F[i+1]*(n-i-2)
  #   = sum i=0 to n-3  F[i+1]*(n-i-2)
  # i=n-3 * 1                 i=0 * n-2
  #                            F[1]*1    n=3
  #                   F[2]*1 + F[1]*2    n=4
  #          F[3]*1 + F[2]*2 + F[1]*3    n=5
  # F[4]*1 + F[3]*2 + F[2]*3 + F[1]*4    n=6
  # F[n-2]
  #
  # 1+1+2+3+5+8 = 20 = 21-1
  #   1+2+3+5+8 = 19 = 21-2
  # sum i=1 to k F[i] = F[k+2]-1
  # sum i=2 to k F[i] = F[k+2]-2
  #
  # S[n] = F[n-2 + 2]-1 + ... + F[1]-1
  #      = F[n] + ... + F[1] - n
  # S[n] = F[n+2] - n - 1

  require Math::NumSeq::Fibonacci;
  my $fib = Math::NumSeq::Fibonacci->new;

  my $formula = sub {
    my ($n) = @_;
    return F($n+2) - $n - 1;
  };
  my $sum = sub {
    my ($n) = @_;
    my $ret = 0;
    foreach my $i (0 .. $n-2) {
      $ret += F($i+1) * ($n-$i-2);
    }
    return $ret;
  };
  foreach my $n (0 .. 20) {
    print $sum->($n),"  ", $formula->($n),"  \n";
  }
  exit 0;
}
