#!/usr/bin/perl -w

# Copyright 2014, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing how the SB tree hull copies out under left
# and right matrices.

use 5.010;
use strict;
use List::Util 'max','min';
use Math::PlanePath::RationalsTree;
use Math::Geometry::Planar;
use Math::NumSeq::Fibonacci;

use FindBin;
use lib $FindBin::Bin;
use MyRationals 'Identity_2x2','mvec_to_pq','SB_left','SB_right';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $depth = 4;

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');
my $fib = Math::NumSeq::Fibonacci->new;

my $points;
foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($depth)) {
  my ($x,$y) = $path->n_to_xy($n);
  push @$points, [$x,$y];
}

my $planar = Math::Geometry::Planar->new;
$planar->points($points);
### $points
if (@$points > 3) { $planar = $planar->convexhull2; }
$points = $planar->points;
### $points

sub draw_hull {
  my ($m) = @_;
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my ($x,$y) = @$p;
    ($x,$y) = mvec_to_pq($m * Math::Matrix->new([$x],[$y]));
    my $join = ($i == 0 ? '' : "--");
    printwrap("$join($x,$y)");
  }
  printwrap("--cycle;\n");
  $Printwrap = 2;
}

my @py_max = $points->[0];
my $y_max = point_sum($py_max[0]);
foreach my $p (@$points) {
  my $y = $p->[1];
  if ($y > $y_max) {
    @py_max = ($p);
    $y_max = $y;
  } elsif ($y == $y_max) {
    push @py_max, $p;
  }
}
if ($py_max[0]->[0] > $py_max[1]->[0]) {
  ($py_max[0],$py_max[1]) = ($py_max[1],$py_max[0]);
}
my $x_max = max(map {$_->[0]} @$points);

my @ps_max = $points->[0];
my $sum_max = point_sum($ps_max[0]);
foreach my $p (@$points) {
  my $sum = point_sum($p);
  if ($sum > $sum_max) {
    @ps_max = ($p);
    $sum_max = $sum;
  } elsif ($sum == $sum_max) {
    push @ps_max, $p;
  }
}
if ($ps_max[0]->[0] > $ps_max[1]->[0]) {
  ($ps_max[0],$ps_max[1]) = ($ps_max[1],$ps_max[0]);
}

my ($Ax,$Ay) = @{$py_max[0]};
my ($Dy,$Dx) = @{$py_max[0]};

my ($Bx,$By) = @{$ps_max[0]};
my ($Cy,$Cx) = @{$ps_max[0]};


#---------------------------------------
# plain

print "  % generated by tools/$FindBin::Script\n";
draw_hull(Identity_2x2());

printwrap("\\coordinate (Xaxis) at ($x_max,0);\n");
printwrap("\\coordinate (Yaxis) at (0,$y_max);\n");
printwrap("\\coordinate (A) at ($Ax,$Ay);\n");
printwrap("\\coordinate (B) at ($Bx,$By);\n");
printwrap("\\coordinate (C) at ($Cx,$Cy);\n");
printwrap("\\coordinate (D) at ($Dx,$Dy);\n");
printwrap("\\fill ($Bx,$By) circle(.7ex);\n");
printwrap("\\fill ($Cx,$Cy) circle(.7ex);\n");

# {
#   my $i = fib_to_i($y_max);
#   my $F = sprintf '$F_{n%+d}$', $i - $depth;
#   $F =~ s/([+-])/{$1}/;
#   printwrap("\\node at (0,$y_max) [left] {$F};\n");
#   printwrap("\\node at ($y_max,0) [below] {$F};\n");
# }

#---------------------------------------
# transformed

(undef, my $tymax) = mvec_to_pq(SB_left() * Math::Matrix->new([0],[$y_max]));
(my $txmax, undef) = mvec_to_pq(SB_right() * Math::Matrix->new([$x_max],[0]));

my ($tAx,$tAy) = mvec_to_pq(SB_left() * Math::Matrix->new([$Ax],[$Ay]));
my ($tBx,$tBy) = mvec_to_pq(SB_left() * Math::Matrix->new([$Bx],[$By]));
my ($tCx,$tCy) = mvec_to_pq(SB_left() * Math::Matrix->new([$Cx],[$Cy]));
my ($tDx,$tDy) = mvec_to_pq(SB_left() * Math::Matrix->new([$Dx],[$Dy]));

# printwrap("\\coordinate (right-graph) at ($x_max*1.3, 0);\n");
printwrap("\\begin{scope}[shift=(right-graph)]\n");

draw_hull(SB_left());
draw_hull(SB_right());

printwrap("%\n");
printwrap("% axes\n");
printwrap("\\draw[dashed] (0,$tBy) -- (0,0) -- ($tBy,0);\n");

printwrap("%\n");
printwrap("\\draw[dashed] (0,0) -- ($tCx,$tCx);\n");
printwrap("\\draw[dashed] ($tymax,$tymax) -- ($tDx,$tDy);\n");
printwrap("\\draw[dashed] ($tymax,$tymax) -- ($tDy,$tDx);\n");

printwrap("%\n");
printwrap("% sides diagonal\n");
printwrap("\\draw[dashed] (0,$tymax) -- (",$tAx-1,",",$tAy-1,");\n");
printwrap("\\draw[dashed] ($tymax,0) -- (",$tAy-1,",",$tAx-1,");\n");

printwrap("%\n");
printwrap("% new M across\n");
printwrap("\\draw[dashed] ($tCx+1,$tCy-1) -- ($tCy-1,$tCx+1);\n");

printwrap("\\fill ($tBx,$tBy) circle(.7ex);\n");
printwrap("\\fill ($tCx,$tCy) circle(.7ex);\n");
printwrap("\\node at ($tAx,$tAy) [above,xshift=-.3em] {A};\n");
printwrap("\\node at ($tBx,$tBy) [above] {B};\n");
printwrap("\\node at ($tCx,$tCy) [right] {C};\n");
printwrap("\\node at ($tDx,$tDy) [right] {D};\n");

printwrap("\\node at ($tAy,$tAx) [right,yshift=-.5ex] {A};\n");
printwrap("\\node at ($tBy,$tBx) [right] {B};\n");
printwrap("\\node at ($tCy,$tCx) [above] {C};\n");
printwrap("\\node at ($tDy,$tDx) [above] {D};\n");

# {
#   my $i = fib_to_i($By);
#   my $F = sprintf '$F_{n%+d}$', $i - $depth;
#   $F =~ s/([+-])/{$1}/;
#   printwrap("\\node at (0,$By) [left] {$F};\n");
#   printwrap("\\node at ($By,0) [below] {$F};\n");
# }
# {
#   my $i = fib_to_i($tBy);
#   my $F = sprintf '$F_{n%+d}$', $i - $depth;
#   $F =~ s/([+-])/{$1}/;
#   printwrap("\\node at (0,$tBy) [left] {$F};\n");
#   printwrap("\\node at ($tBy,0) [below] {$F};\n");
# }

printwrap("\\end{scope}\n");


#---------------------------------------

sub point_sum {
  my ($p) = @_;
  return $p->[0] + $p->[1];
}

sub fib_to_i {
  my ($value) = @_;
  foreach my $i (0 .. 100) {
    if ($fib->ith($i) == $value) { return $i; }
  }
  die;
}
exit 0;
