#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing a minimum area rectangle around the SB tree
# points, being all points rows 0 to $depth.

use 5.010;
use strict;
use List::Util 'max','min';
use Math::Geometry::Planar;
use Math::NumSeq::Fibonacci;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyRationals 'F','planar_isboundary';
use MyPrintwrap ':all';


my $depth = 5;

print "  % generated by tools/$FindBin::Script\n";
print "  % depth d=$depth\n";

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');

my $points;
my $x_max = 0;
my $y_max = 0;
foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($depth)) {
  my ($x,$y) = $path->n_to_xy($n);
  push @$points, [$x,$y];
  $x_max = max($x_max, $x);
  $y_max = max($x_max, $y);
}

my $planar = Math::Geometry::Planar->new;
$planar->points($points);
if (@$points > 3) { $planar = $planar->convexhull2; }
$points = $planar->points;
my $i_mid = $#$points/2;
### $points

{
  $Printwrap = 2;
  printwrap("\\draw[line join=round] ");
  $Printwrap += 2;
  my $join = '';
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my ($x,$y) = @$p;
    printwrap("$join($x,$y)");
    $join = '--';
  }
  printwrap("--cycle;\n");
  printwrap_newline();
}

my $corner_west;
my $corner_north;
my $corner_south;
my $corner_east;
foreach my $i (0 .. $#$points) {
  my $p1 = $points->[$i-1];
  my $p2 = $points->[$i];
  my ($x1,$y1) = @$p1;
  my ($x2,$y2) = @$p2;
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;
  if ($dx == $dy && $y1 > $x1) {
    print "  % side $x1,$y1 to $x2,$y2\n";
    $corner_west = PerpendicularFoot([$p1,$p2,
                                      [1,1]]);
    $corner_north = PerpendicularFoot([$p1,$p2,
                                       [F($depth+1),F($depth+2)]]);

    $corner_south = PerpendicularFoot([$corner_west,[1,1],
                                       [F($depth+2),F($depth)]]);
    $corner_east = PerpendicularFoot([$corner_north,
                                      [F($depth+1),F($depth+2)],
                                      [F($depth+2),F($depth)]]);
    last;
  }
}

my ($north_x,$north_y) = @$corner_north;
my ($south_x,$south_y) = @$corner_south;
my ($east_x,$east_y) = @$corner_east;
my ($west_x,$west_y) = @$corner_west;

printf "  \\draw (%.1f,%.1f)--(%.1f,%.1f)--(%.1f,%.1f)--(%.1f,%.1f)--cycle;\n",
  @$corner_west,
  @$corner_north,
  @$corner_east,
  @$corner_south;


{
  my $count_boundary = 0;
  my $rectpoly = Math::Geometry::Planar->new;
  $rectpoly->points([$corner_west,$corner_north,$corner_east,$corner_south]);
  $Printwrap = 2;
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my $style = '[small dot]';
    if (planar_isboundary($rectpoly, $p)) {
      $count_boundary++;
      $style = '[big dot]';
    }
    my ($x,$y) = @$p;
    printwrap(" \\fill$style ($x,$y) circle;");
  }
  printwrap_newline();
  printwrap(" % $count_boundary points on boundary\n");
}

print "  % end generated\n";
exit 0;
